
;**************************************************************************
;*                    Atmel I2C Programmer                                *
;**************************************************************************
;**************************************************************************
;*          v 1.31 - 08.08.2013                                           *
;**************************************************************************
;*          Hardw.: PIC16F887 used                                        *
;*          OSC.......: Internal osc. used     POWER.....:   18V extern.  *
;**************************************************************************


;------------------------------------------------------
;device addresses:
;	EEPROM:			1010  0  0  0 x
;------------------------------------------------------


;on board 24FC512:
;	A0 -- GND
;	A1 -- GND
;	A2 -- GND
;	WP -- GND

;	SDA -- SDA (RC4)
;	SCL -- SCL (RC3)

;keyboard
;	kbdclk  -- RB0
;	kbddata -- RB1



;Atmel programming procedure is based on description of external programming for ATtiny2313A from Atmel
;a small part has been added for ATtiny48/88
;only difference in the procedure are different page size and number of pages for the different microcontrollers

;------------------------------------------------
;------------------------------------------------
;following microcontrollers are supported:	|
;------------------------------------------------
;	Name		Internal Code           |
;------------------------------------------------
;	ATtiny2313	01
;	ATtiny2313A	01
;	ATtiny4313	02
;	ATtiny26	08
;	ATtiny261A	06
;	ATtiny461A	04
;	ATtiny861A	07
;	ATtiny48	05
;	ATtiny88	03
;
;	ATmega48A	11
;	ATmega48PA	12
;	ATmega88A	13
;	ATmega88PA	14
;	ATmega168A	15
;	ATmega168PA	16
;	ATmega328	17
;	ATmega328P	18
;------------------------------------------------
;------------------------------------------------

 INCLUDE "P16F887.INC"


_U001			     EQU     B'1110000011000100'	;see Page 206 of Datasheet
_U002			     EQU     B'1111111111111111'	;see Page 207 of Datasheet

;	__CONFIG	_CONFIG1, _DEBUG_OFF & _PLL_EN & _BORV_2_5 & _BOR_OFF & _CP_OFF & _MCLR_DIS & _PWRT_DIS & _WDT_OFF & _INTRC_OSC_NOCLKOUT
;	__CONFIG	_CONFIG2, _VCAP_DIS


	__CONFIG	_CONFIG1, _U001

	__CONFIG	_CONFIG2, _U002


;==========================================================================
;       Variable Definition
;==========================================================================
TIMER1		EQU	H'20'		;delay routine
TIMER2		EQU	H'21'		; "	"	"

;interrupt
W_TEMP		EQU	H'22'
STATUS_TEMP	EQU	H'23'
PCLATH_TEMP	EQU	H'24'
data_tmp	EQU	H'25'
bus_D		EQU	H'26'
cnt1		EQU	H'27'
cnt2		EQU	H'28'
v_eeprom_sel	EQU	H'29'
kbd_cnt1	EQU	H'2A'
kbd_buffer	EQU	H'2B'		;keyboard byte buffer
kbd_bb		EQU	H'2c'		;keyboard bit buffer
disp_pos	EQU	H'2D'
c_tmp		EQU	H'2E'

v_vt		EQU	H'2F'
;----------------------------------------------------------------------
v_fuse_mod	EQU	H'30'		;fuse info modified
;----------------------------------------------------------------------
v_tmp5		EQU	H'31'		;general usage
v_tmp6		EQU	H'32'		;general usage
;---------------------------------------------------------------------
v_att_pga	EQU	H'33'		;address within a page


v_att_fl1	EQU	H'34'
v_att_fh1	EQU	H'35'
v_att_fl2	EQU	H'36'
v_att_fh2	EQU	H'37'
v_att_fl3	EQU	H'38'
v_att_fh3	EQU	H'39'
v_att_fl	EQU	H'3A'		;flash byt low
v_att_fh	EQU	H'3B'		;flash byt high
;
v_att_s0	EQU	H'3C'		;ATtiny signature byte 0
v_att_s1	EQU	H'3D'		;ATtiny signature byte 1
v_att_s2	EQU	H'3E'		;ATtiny signature byte 2
;
v_att_add	EQU	H'3F'		;ATtiny address to read/write

v_att_cnt	EQU	H'40'
v_att_pgs	EQU	H'41'		;pages to be written
v_att_pgs_w	EQU	H'42'		;written pages

v_att_chip	EQU	H'43'		;identified AT microcontroler
					;01 = ATtiny2313(A)
					;02 = ATtiny4313
					;03 = ATtiny88
					;FE = not known
					;FF = not inserted / not known
;---------------------------------------------------------------------
v_fuse_e	EQU	H'44'
v_fuse_h	EQU	H'45'
v_fuse_l	EQU	H'46'
v_lock_b	EQU	H'47'
;---------------------------------------------------------------------
v_pcf_add	EQU	H'48'		;PCF8574 I2C address
;
kaf		EQU	H'49'		;keyboard activity flag
;
vs_d0		EQU	H'4A'
vs_d1		EQU	H'4B'
vs_d2		EQU	H'4C'
vs_d3		EQU	H'4D'
;
kbd_out		EQU	H'4E'
kbd_parity	EQU	H'4F'

b_ax		EQU	H'50'
tmp_ot		EQU	H'51'

s_eeprom	EQU	H'52'

v_d0		EQU	H'53'		;EEPROM data - relative address 0
v_d1		EQU	H'54'		;EEPROM data - relative address 1
v_d2		EQU	H'55'		;EEPROM data - relative address 2
v_d3		EQU	H'56'		;EEPROM data - relative address 3
v_d4		EQU	H'57'		;EEPROM data - relative address 4
v_d5		EQU	H'58'		;EEPROM data - relative address 5

v_ad0		EQU	H'59'		;EEPROM address - lowest 8 bits
v_ad1		EQU	H'5A'		;EEPROM address - highest bits

v_byte		EQU	H'5B'		;data to be written to PCF8574
v_temp1		EQU	H'5C'		;temperature from TMP100 (MSB)
v_temp2		EQU	H'5D'		;temperature from TMP100 (LSB)

v_tempx		EQU	H'5E'		;temperature in dec
v_LED		EQU	H'5F'		;temperature for 8 LEDs

v_blink		EQU	H'60'

;----------------------------------------------
;----------------------------------------------
v_mll		EQU	H'61'		;Processor memory location LSB
v_mlh		EQU	H'62'		;Processor memory location MSB
;----------------------------------------------
;----------------------------------------------
v_pages		EQU	H'63'		;number of pages to be flashed
v_page_size	EQU	H'64'		;Page size
;----------------------------------------------
v_b_in		EQU	H'65'		;HEX input field
v_comm		EQU	H'66'		;command
v_cursor	EQU	H'67'		;cursor position in LCD
;----------------------------------------------
b_ax_h		EQU	H'68'
;----------------------------------------------

;FREE !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
;FREE !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
;FREE !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!

;----------------------------------------------
;----------------------------------------------
v_tmp1		EQU	H'75'	;temp variable - general usage

v_rtc_t1	EQU	H'76'
v_rtc_t2	EQU	H'77'

v_att_add_h	EQU	H'78'
v_cycles	EQU	H'79'
;--------------------------------------------------
;-------------- registers BANK2 -------------------
;--------------------------------------------------
v_m01l		EQU	H'110'
v_m01h		EQU	H'111'
v_m02l		EQU	H'112'
v_m02h		EQU	H'113'
v_m03l		EQU	H'114'
v_m03h		EQU	H'115'
v_m04l		EQU	H'116'
v_m04h		EQU	H'117'
v_m05l		EQU	H'118'
v_m05h		EQU	H'119'
v_m06l		EQU	H'11A'
v_m06h		EQU	H'11B'
v_m07l		EQU	H'11C'
v_m07h		EQU	H'11D'
v_m08l		EQU	H'11E'
v_m08h		EQU	H'11F'

;==========================================================================

		ORG	0		;Reset vector address
		GOTO	RESET		;goto RESET routine when boot.

;XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
;XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
;----------------------------------------------------------------------------
; INTERRUPT SERVICE ROUTINE -------------------------------------------------
;----------------------------------------------------------------------------
;XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
;XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX

		ORG	004
;save important variables
		MOVWF	W_TEMP			;Copy W to W_TEMP register
		SWAPF	STATUS,W		;Swap status to be saved into W

;Swaps are used because they do not affect the status bits

		BANKSEL	STATUS_TEMP		;Select regardless of current bank
		MOVWF	STATUS_TEMP		;Copy status to bank zero STATUS_TEMP register
		MOVF	PCLATH,W		;Copy PCLATH to W register
		MOVWF	PCLATH_TEMP		;Copy W register to PCLATH_TEMP

		bcf	INTCON,GIE

		PAGESEL	PAGE0

		goto	int_serv
;----------------------------------------------------------------------------
;----------------------------------------------------------------------------
;
;	       **********************************
;              **  RESET :  main boot routine  **
;              **********************************

RESET		nop
PAGE0
;
		BSF	STATUS,RP0	;Switch to register bank 1
		MOVLW	B'01011111'	;Set PIC options (See datasheet).
		MOVWF	OPTION_REG	;Write the OPTION register.
		bcf	OPTION_REG,T0CS	;set-up TIMER0 as timer
		bsf	OPTION_REG,PSA	;prescaler NOT for TIMER0
		;
		CLRF	INTCON		;Disable interrupts
		clrf	IOCB
		bsf	IOCB,D'000'	;enable interrupt on change for RB0 only

		MOVLW	B'11000000'	;PORTA
		MOVWF	TRISA

		MOVLW	B'11111111'	;RB0 = keyboard clock; RB1 = kbd data
		MOVWF	TRISB

		MOVLW	B'00011000'	;PORTC
		MOVWF	TRISC

		MOVLW	B'00110000'	;PORTD = input/output: RD0-RD3 - LCD; RD4-RD7 = PT2272-L2/A8-A9, D0-D1
		MOVWF	TRISD

		MOVLW	B'11111000'	;PORTE = output; RE3 = input
		MOVWF	TRISE

		movlw	B'00000011'
		movwf	WPUB

		bsf	STATUS,RP0
		bsf	STATUS,RP1
		clrf	ANSEL
		clrf	ANSELH

		bcf	STATUS,RP0	;Switch Back to reg. Bank 0
		bcf	STATUS,RP1

;----------------------------------------------------------------------------
;set up I2C BUS for communication as a master
		movlw	B'00101000'		;see Datasheet Page 177
;			bit 3-0 SSPM<3:0>: Synchronous Serial Port Mode Select bits
;			1000 = I2C Master mode, clock = FOSC / (4 * (SSPADD+1))
		movwf	SSPCON

;----------------------------------------------------------------------------


; XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
; XXXXXXXXXX       initialization  XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
; XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX

init_main	clrf	PORTC
		clrf	PORTA
		clrf	PORTB
;
		clrf	kaf

		clrf	PIE1
		clrf	PIE2

		bsf	INTCON,GIE
		bsf	INTCON,PEIE

		bsf	PIE1,SSPIE
;
		movlw	H'FF'
		movwf	v_att_s0
		movwf	v_att_s1
		movwf	v_att_s2
;
		bsf	PORTC,D'005'		;connect EEPROM to internal I2C bus

		bsf	v_eeprom_sel,D'000'	;64kb EEPROM selected
;-----------------------------------------------------------------------------
;init display - SCREEN1
;
		PAGESEL	PAGE1
		call	disp_init
		call	w_release
		PAGESEL	PAGE0
;-------------------------------------------------------------------------------
;--- init phase is finished now ------------------------------------------------
;-------------------------------------------------------------------------------
		call	dr2
		call	dr2
		call	dr2
		call	dr2
		call	dr2
		call	dr2
		call	dr2
;------------------------------------------------------------------------------
;------------------------------------------------------------------------------
;
		PAGESEL	PAGE1
		call	clear_disp
		call	w_wkbd
		PAGESEL	PAGE0

;--- test ----
led_setup	movlw	H'ED'
		movwf	kbd_out
		call	send_ck
		call	dr01s			;ignore the ack from kbd and wait a little bit
		movlw	B'00000010'
		movwf	kbd_out
		call	send_ck
		bsf	PORTC,D'007'
		call	dr01


;XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
;XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
;-----------------------------------------------------------------------------
;                          Main entry screen
;-----------------------------------------------------------------------------
;XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
;XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
m001		nop
		PAGESEL	PAGE1
		call	disp_clr1
		call	disp_clr2
		call	w_m001
		PAGESEL	PAGE0

		bsf	PORTC,D'005'		;connect EEPROM to internal I2C bus
		bcf	PORTC,D'006'		;switch the LED off

		movlw	B'01000000'
		movwf	v_pcf_add		;default I2C address for PCF8574

m002		clrf	kbd_buffer
		call	get_datafk

		movf	kbd_buffer,0
		xorlw	H'4D'		;P = PICkit2 connect
		btfsc	STATUS,Z
		goto	pickit2

		movf	kbd_buffer,0
		xorlw	H'1C'		;A = ATtiny2313A programmer
		btfsc	STATUS,Z
		goto	att_call

		movf	kbd_buffer,0
		xorlw	H'24'		;E = EEPROM read
		btfsc	STATUS,Z
		goto	eeprom_read

		goto	m002

;XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
;XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
;-----------------------------------------------------------------------------
;                          PICkit2 connect
;-----------------------------------------------------------------------------
;XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
;XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX

pickit2		nop
		PAGESEL	PAGE1
		call	disp_clr1
		call	disp_clr2
		call	w_pickit2
		PAGESEL	PAGE0

		bcf	PORTC,D'005'		;connect EEPROM to external I2C bus
		bsf	PORTC,D'006'		;switch the LED on

		clrf	v_blink

;keaboard test ----------------------------------------
pk2_l		call	d2

		call	pk_blink

		movlw	H'FE'
		movwf	kbd_out
		call	send_ck

waitclk		btfss	PORTB,D'000'
		goto	waitclk

waitdat		btfss	PORTB,D'001'
		goto	waitdat

waitclk2	btfsc	PORTB,D'000'
		goto	waitclk2

		clrf	kbd_buffer
		call	get_datafk
		movf	kbd_buffer,0
		xorlw	H'76'		;ESC -> exit
		btfsc	STATUS,Z
		goto	m001

		goto	pk2_l

;------------------------------------------------------
;keaboard test ----------------------------------------
pk2_l2		call	d2
		call	pk_blink

		clrf	kbd_buffer
		call	get_datafk

		movf	kbd_buffer,0
		xorlw	H'76'		;ESC -> exit
		btfsc	STATUS,Z
		goto	m001

		goto	pk2_l2

;---------------------------------------------------------------------------------------------------
;---------------------------------------------
pk_blink	incf	v_blink,1
		btfss	v_blink,D'001'
		goto	pkw42
		bsf	PORTC,D'006'		;switch the LED on
		return

pkw42		bcf	PORTC,D'006'		;switch the LED off
		return
;---------------------------------------------
;---------------------------------------------
;---------------------------------------------------------------------------------------------------
;---------------------------------------------------------------------------------------------------
;XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
;XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
;-----------------------------------------------------------------------------
;                          ATtiny2313A
;-----------------------------------------------------------------------------
;XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
;XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
att_call	nop
		PAGESEL	PAGE2
		goto	att_start
		PAGESEL	PAGE0

;XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
;XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
;-----------------------------------------------------------------------------
;                          EEPROM Read Screen
;-----------------------------------------------------------------------------
;XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
;XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
eeprom_read	clrf	v_ad0
		clrf	v_ad1

		PAGESEL	PAGE1
		call	disp_clr1
		call	disp_clr2
		PAGESEL	PAGE0

		movlw	B'10100000'		;I2C device address
		movwf	v_pcf_add

eep_l2r		call	eeprom_d_r		;data read from address

eep_l2		nop
		PAGESEL	PAGE1
		movlw	B'10000000'		;row: 1 / col: 0
		call	w_eeprom_data
		PAGESEL	PAGE0

		call	eeprom_inc_ad
		call	eeprom_d_rn		;read next
		PAGESEL	PAGE1
		movlw	B'11000000'		;row: 2 / col: 0
		call	w_eeprom_data
		PAGESEL	PAGE0


eep_l1		clrf	kbd_buffer
		call	get_datafk

		movf	kbd_buffer,0
		xorlw	H'5A'		;ENTER = forward
		btfsc	STATUS,Z
		goto	eep_fw1

		movf	kbd_buffer,0
		xorlw	H'74'		;-> = forward
		btfsc	STATUS,Z
		goto	eep_fw1

		movf	kbd_buffer,0
		xorlw	H'75'		;^ (arrow up) = page forward
		btfsc	STATUS,Z
		goto	eep_fw2

		movf	kbd_buffer,0
		xorlw	H'6B'		;<- = backward
		btfsc	STATUS,Z
		goto	eep_bck1

		movf	kbd_buffer,0
		xorlw	H'72'		;v (arrow down) = page back
		btfsc	STATUS,Z
		goto	eep_bck2

		movf	kbd_buffer,0
		xorlw	H'76'		;ESC = back to EEPROM selection
		btfsc	STATUS,Z
		goto	eeprom_back

		goto	eep_l1


eep_fw1		call	eeprom_inc_ad
		call	eeprom_d_rn		;read next
		goto	eep_l2

eep_fw2		call	eep_i2c_close
		call	eeprom_inc_pg
		goto	eep_l2r

eep_bck1	call	eep_i2c_close
		call	eeprom_dec_ad
		goto	eep_l2r

eep_bck2	call	eep_i2c_close
		call	eeprom_dec_pg
		goto	eep_l2r

;--------------------------------------------------------------------
eeprom_back	nop
		PAGESEL PAGE1
		call	disp_clr1
		call	disp_clr2
		PAGESEL PAGE0

;close the communication
		call	eep_i2c_close
		goto	m001
;-----------------------------------------------------------------------------
;-----------------------------------------------------------------------------
;-----------------------------------------------------------------------------
eeprom_inc_pg	incf	v_ad1,1
		return
;-----------------------------------------------------------------------------
eeprom_dec_pg	decf	v_ad1,1
		return
;-----------------------------------------------------------------------------
;increment the address to be displayed - EEPROM data read
;EEPROM pointer is incremented by 6 (v_ad1 -- v_ad0)
eeprom_inc_ad	movlw	D'006'
eeprom_inc_adn	movwf	TIMER1
eaa1		incf	v_ad0,1
		btfss	STATUS,Z
		goto	eaa4
		clrf	v_ad0
		incf	v_ad1,1
eaa4		decfsz	TIMER1,1
		goto	eaa1

		return
;-----------------------------------------------------------------------------
;decrement the address to be displayed - EEPROM data read
;EEPROM pointer is decremented by 12 (v_ad1 -- v_ad0)
eeprom_dec_ad	movlw	D'018'
		movwf	TIMER1
ebb1		decf	v_ad0,1
		movlw	H'FF'
		xorwf	v_ad0,0
		btfss	STATUS,Z
		goto	ebb4
		decf	v_ad1,1
ebb4		decfsz	TIMER1,1
		goto	ebb1

		return
;-----------------------------------------------------------------------------
;-----------------------------------------------------------------------------
;-----------------------------------------------------------------------------




;-----------------------------------------------------------------------------
;-----------------------------------------------------------------------------
; EEPROP read routines
;-----------------------------------------------------------------------------
;-----------------------------------------------------------------------------
eeprom_d_r	nop
;
		call	eep_i2c_open_r
		call	eeprom_d_rn
		return
;-----------------------------------------
eeprom_d_rn	nop

		call	eep_i2c_read
		movf	kbd_buffer,0
		movwf	v_d0

		call	eep_i2c_read
		movf	kbd_buffer,0
		movwf	v_d1

		call	eep_i2c_read
		movf	kbd_buffer,0
		movwf	v_d2

		call	eep_i2c_read
		movf	kbd_buffer,0
		movwf	v_d3

		call	eep_i2c_read
		movf	kbd_buffer,0
		movwf	v_d4

		call	eep_i2c_read
		movf	kbd_buffer,0
		movwf	v_d5

		return
;-----------------------------------------------------------------------------

;-----------------------------------------------------------------------------
;xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx
;xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx
;xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx
;
; I2C communication routines
;
;xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx
;xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx
;xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx



;-----------------------------------------------------------------------------
;-----------------------------------------------------------------------------
;XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
;-----------------------------------------------------------------------------
; send data to PORTA
;-----------------------------------------------------------------------------
;-----------------------------------------------------------------------------
pcf_i2c_write	bsf	STATUS,RP0
		movlw	D'000'
		movwf	TRISA
		bcf	STATUS,RP0

		movf	v_byte,0
		movwf	PORTA
		return
;-----------------------------------------------------------------------------
;XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
;-----------------------------------------------------------------------------
; read data from PORTA
;-----------------------------------------------------------------------------
;-----------------------------------------------------------------------------
pcf_i2c_read	bsf	STATUS,RP0
		movlw	D'255'
		movwf	TRISA
		bcf	STATUS,RP0

		movf	PORTA,0
		movwf	v_byte
		return

;-----------------------------------------------------------------------------
;-----------------------------------------------------------------------------
; EEPROP I2C communication routines
;-----------------------------------------------------------------------------
;-----------------------------------------------------------------------------

;-----------------------------------------------------------------------------
; I2C EEPROM - open the device for reading
;-----------------------------------------------------------------------------
eep_i2c_open_r	bsf	STATUS,RP0
		movlw	B'00111111'
		movwf	SSPADD
		clrf	SSPSTAT
		bcf	STATUS,RP0

;2nd step - Start condition
		bsf	STATUS,RP0
		bsf	SSPCON2, SEN
		bcf	STATUS,RP0

		call	d55
		bcf	PIR1, SSPIF
;
;Step 3A: addressing the EEPROM device

;EEPROM = 1010; A2 = A1 = A0 = 0; R/-W = 0 (write)
		movlw	B'10100000'		;I2C device address -- only one EEPROM used in this device; the address can be hard defined on this place
		movwf	v_pcf_add

		movf	v_pcf_add,0		;get the device address
		andlw	B'11111110'		;we want to write - last bit needs to be "0"
		movwf	SSPBUF

		btfsc	SSPCON,WCOL
		goto	wc_error

		bcf	PIR1, SSPIF
		call	d55
		bcf	PIR1, SSPIF

;Step 3B: word address (to be read)

		btfss	v_eeprom_sel,D'000'
		goto	x111
;-----------------------------------------------
;this part is only for 64k EEPROM --------------
		movf	v_ad1,0
		movwf	SSPBUF

		btfsc	SSPCON,WCOL
		goto	wc_error

		bcf	PIR1, SSPIF
		call	d55
		bcf	PIR1, SSPIF
;-----------------------------------------------
x111		movf	v_ad0,0
		movwf	SSPBUF

		btfsc	SSPCON,WCOL
		goto	wc_error

		bcf	PIR1, SSPIF
		call	d55
		bcf	PIR1, SSPIF

;Step 3C: 2nd Start condition
		bsf	STATUS,RP0
		bsf	SSPCON2, SEN
		bcf	STATUS,RP0

		call	d55
		bcf	PIR1, SSPIF

;Step 3D: addressing the EEPROM device

		movf	v_pcf_add,0
		iorlw	B'00000001'		;we want to read - last bit needs to be "1"
		movwf	SSPBUF

		btfsc	SSPCON,WCOL
		goto	wc_error

		bcf	PIR1, SSPIF
		call	d55
		bcf	PIR1, SSPIF

		return
;-----------------------------------------------------------------------------

;-----------------------------------------------------------------------------
; I2C EEPROM read - next byte; "the file needs to be opened before"
;-----------------------------------------------------------------------------
eep_i2c_read	bsf	STATUS,RP0
		bsf	SSPCON2, RCEN
		bcf	STATUS,RP0
		
		call	d55
		bcf	PIR1, SSPIF

		movf	SSPBUF,0		;received byte
		movwf	kbd_buffer

;
		bsf	STATUS,RP0
		bcf	SSPCON2, ACKDT		;ACK
		bsf	SSPCON2, ACKEN
		bcf	STATUS,RP0
		
		call	d55
		bcf	PIR1, SSPIF
		return
;-----------------------------------------------------------------------------

;-----------------------------------------------------------------------------
; I2C EEPROM close the file
;-----------------------------------------------------------------------------
eep_i2c_close	bsf	STATUS,RP0
		bsf	SSPCON2, RCEN
		bcf	STATUS,RP0
		
		call	d55
		bcf	PIR1, SSPIF

		movf	SSPBUF,0		;received byte
		movwf	kbd_buffer

;2nd: send NO-ACK

		bsf	STATUS,RP0
		bsf	SSPCON2, ACKDT		;NO ACK
		bsf	SSPCON2, ACKEN
		bcf	STATUS,RP0
		
		call	d55
		bcf	PIR1, SSPIF

;3rd: send STOP condition
		bsf	STATUS,RP0
		bsf	SSPCON2, PEN
		bcf	STATUS,RP0
		
		bcf	PIR1, SSPIF

		call	d55

		return
;-----------------------------------------------------------------------------


;XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
;XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
;-----------------------------------------------------------------------------
;                          smoke MAIN SCREEN
;-----------------------------------------------------------------------------
;XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
;XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX


smc001		nop
		PAGESEL	PAGE1
		movlw	B'10000000'		;row: 1 / col: 0
		call	send_comm

		call	wm_0
		call	wm_0
		call	wm_0
		call	wm_dvojbodka

;now we have to get some data from EEPROM
;description of I2C Master Mode starts in Datasheet on Page 189

		call	wm_otaznik
		call	wm_otaznik
		PAGESEL	PAGE0

;summary of used I2C registers:
;	SSPBUF - data / address buffer			[bank0]
;	SSPCON						[bank0]

;	SSPCON2						[bank1]
;	SSPSTAT						[bank1]
;	SSPADD						[bank1] - for baud rate generator

;

		call	smk_s13

smk_s45		nop
		PAGESEL	PAGE1
		movlw	B'10000100'		;row: 1 / col: 4
		call	send_comm
		PAGESEL	PAGE0

;Steps 4 & 5
		call	smk_read
		call	fin002
		call	smk_read
		call	fin002
		call	smk_read
		call	fin002
		call	smk_read
		call	fin002

		goto	fin001

;-------------------------------------------------------------------
;write received byte
;
fin001		clrf	kbd_buffer
		call	get_datafk

		movf	kbd_buffer,0
		xorlw	H'5A'		;ENTER = read next
		btfsc	STATUS,Z
		goto	smk_s45

		goto	fin001

wc_error	nop
		PAGESEL	PAGE1
		call	wm_W
		call	wm_C
		call	wm_E
		PAGESEL	PAGE0
		goto	fin001


;-----------------------------------------------------------------------
;Step 4: receive the data
smk_read	bsf	STATUS,RP0
		bsf	SSPCON2, RCEN
		bcf	STATUS,RP0
		
		call	d55
		bcf	PIR1, SSPIF

		movf	SSPBUF,0		;received byte
		movwf	kbd_buffer

;Step 5: send ACK

		bsf	STATUS,RP0
		bcf	SSPCON2, ACKDT
		bsf	SSPCON2, ACKEN
		bcf	STATUS,RP0
		
		call	d55
		bcf	PIR1, SSPIF
		return
;-----------------------------------------
fin002		nop
		PAGESEL	PAGE1
		call	write_hex
		call	wm_space
		PAGESEL	PAGE0

		return

;-----------------------------------------------------------------------------
;-----------------------------------------------------------------------------
;start communication with EEPROM
;-----------------------------------------------------------------------------
;-----------------------------------------------------------------------------
smk_s13		bsf	STATUS,RP0
		movlw	B'00111111'
		movwf	SSPADD
		clrf	SSPSTAT
		bcf	STATUS,RP0

;1st step - preparation
		bcf	PORTA,D'000'		;A0 = 0
		bcf	PORTA,D'001'		;A1 = 0
		bcf	PORTA,D'002'		;A2 = 0

		bcf	PORTC,D'005'		;WP = 0 (no write protect)


;2nd step - Start condition
		bsf	STATUS,RP0
		bsf	SSPCON2, SEN
		bcf	STATUS,RP0

		call	d55
		bcf	PIR1, SSPIF
;
;Step 3A: addressing the EEPROM device

		movlw	B'10100001'		;EEPROM = 1010; A2 = A1 = A0 = 0; R/-W = 0 (read)
		movwf	SSPBUF

		btfsc	SSPCON,WCOL
		goto	wc_error

		bcf	PIR1, SSPIF
		call	d55
		bcf	PIR1, SSPIF

		return
;---------------------------------------------------------------------------------------
;---------------------------------------------------------------------------------------
;Step 6: Stop condition

smk_s6		bsf	STATUS,RP0
		bsf	SSPCON2, PEN
		bcf	STATUS,RP0
		
		bcf	PIR1, SSPIF

		call	d55

		return
;XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
;---------------------------------------------------------------------------------------
;---------------------------------------------------------------------------------------
;XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
;---------------------------------------------------------------------------------------
;---------------------------------------------------------------------------------------
;XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX

;-------------------------------------------------
hi_leave_esc	movlw	H'01'
		movwf	v_comm
		call	dr77
		return

hi_leave_enter	movlw	H'02'
		movwf	v_comm
		call	dr77
		return

hi_leave_F10	movlw	H'03'
		movwf	v_comm
		call	dr77
		return
;-------------------------------------------------
;-------------------------------------------------
sa_key		movlw	H'00'
		movwf	v_comm

		PAGESEL PAGE1
		movf	tmp_ot,0
		call	send_comm
		PAGESEL PAGE0

sa_key3		btfss	PORTB,D'000'
		goto	sa_key2
		goto	sa_key3

;
sa_key2		clrf	kbd_buffer
		call	get_datafk

		movf	kbd_buffer,0
		xorlw	H'76'		;ESC
		btfsc	STATUS,Z
		goto	hi_leave_esc

		movf	kbd_buffer,0
		xorlw	H'5A'		;ENTER
		btfsc	STATUS,Z
		goto	hi_leave_enter

		movf	kbd_buffer,0
		xorlw	H'09'		;F10
		btfsc	STATUS,Z
		goto	hi_leave_F10


		movf	kbd_buffer,0
		xorlw	H'45'		;0
		btfsc	STATUS,Z
		goto	set_av0

		movf	kbd_buffer,0
		xorlw	H'16'		;1
		btfsc	STATUS,Z
		goto	set_av1

		movf	kbd_buffer,0
		xorlw	H'1E'		;2
		btfsc	STATUS,Z
		goto	set_av2

		movf	kbd_buffer,0
		xorlw	H'26'		;3
		btfsc	STATUS,Z
		goto	set_av3

		movf	kbd_buffer,0
		xorlw	H'25'		;4
		btfsc	STATUS,Z
		goto	set_av4

		movf	kbd_buffer,0
		xorlw	H'2E'		;5
		btfsc	STATUS,Z
		goto	set_av5

		movf	kbd_buffer,0
		xorlw	H'36'		;6
		btfsc	STATUS,Z
		goto	set_av6

		movf	kbd_buffer,0
		xorlw	H'3D'		;7
		btfsc	STATUS,Z
		goto	set_av7

		movf	kbd_buffer,0
		xorlw	H'3E'		;8
		btfsc	STATUS,Z
		goto	set_av8

		movf	kbd_buffer,0
		xorlw	H'46'		;9
		btfsc	STATUS,Z
		goto	set_av9

		movf	kbd_buffer,0
		xorlw	H'1C'		;A
		btfsc	STATUS,Z
		goto	set_avA

		movf	kbd_buffer,0
		xorlw	H'32'		;B
		btfsc	STATUS,Z
		goto	set_avB

		movf	kbd_buffer,0
		xorlw	H'21'		;C
		btfsc	STATUS,Z
		goto	set_avC

		movf	kbd_buffer,0
		xorlw	H'23'		;D
		btfsc	STATUS,Z
		goto	set_avD

		movf	kbd_buffer,0
		xorlw	H'24'		;E
		btfsc	STATUS,Z
		goto	set_avE

		movf	kbd_buffer,0
		xorlw	H'2B'		;F
		btfsc	STATUS,Z
		goto	set_avF

		goto	sa_key3
;-----------------------------------
set_av0		movlw	'0'
		movwf	b_ax
		movlw	H'00'
		movwf	b_ax_h
		call	dr77
		return

set_av1		movlw	'1'
		movwf	b_ax
		movlw	H'01'
		movwf	b_ax_h
		call	dr77
		return

set_av2		movlw	'2'
		movwf	b_ax
		movlw	H'02'
		movwf	b_ax_h
		call	dr77
		return

set_av3		movlw	'3'
		movwf	b_ax
		movlw	H'03'
		movwf	b_ax_h
		call	dr77
		return

set_av4		movlw	'4'
		movwf	b_ax
		movlw	H'04'
		movwf	b_ax_h
		call	dr77
		return

set_av5		movlw	'5'
		movwf	b_ax
		movlw	H'05'
		movwf	b_ax_h
		call	dr77
		return

set_av6		movlw	'6'
		movwf	b_ax
		movlw	H'06'
		movwf	b_ax_h
		call	dr77
		return

set_av7		movlw	'7'
		movwf	b_ax
		movlw	H'07'
		movwf	b_ax_h
		call	dr77
		return

set_av8		movlw	'8'
		movwf	b_ax
		movlw	H'08'
		movwf	b_ax_h
		call	dr77
		return

set_av9		movlw	'9'
		movwf	b_ax
		movlw	H'09'
		movwf	b_ax_h
		call	dr77
		return

set_avA		movlw	'A'
		movwf	b_ax
		movlw	H'0A'
		movwf	b_ax_h
		call	dr77
		return

set_avB		movlw	'B'
		movwf	b_ax
		movlw	H'0B'
		movwf	b_ax_h
		call	dr77
		return

set_avC		movlw	'C'
		movwf	b_ax
		movlw	H'0C'
		movwf	b_ax_h
		call	dr77
		return

set_avD		movlw	'D'
		movwf	b_ax
		movlw	H'0D'
		movwf	b_ax_h
		call	dr77
		return

set_avE		movlw	'E'
		movwf	b_ax
		movlw	H'0E'
		movwf	b_ax_h
		call	dr77
		return

set_avF		movlw	'F'
		movwf	b_ax
		movlw	H'0F'
		movwf	b_ax_h
		call	dr77
		return
;------------------------------------
;-------------------------------------------------------------------------
;XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
;XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX

;		*********************************************
;		*               delay routine               *
;		*********************************************

;------------------------------------
d7		MOVLW   D'127'
		MOVWF   TIMER2
DEL_LOOP71	MOVLW   D'255'
		MOVWF   TIMER1

DEL_LOOP72	DECFSZ  TIMER1,F
		GOTO    DEL_LOOP72
		DECFSZ  TIMER2,F
		GOTO    DEL_LOOP71
		return

;------------------------------------

d2		MOVLW   D'255'
		MOVWF   TIMER2
DEL_LOOP1	MOVLW   D'255'
		MOVWF   TIMER1

DEL_LOOP2	DECFSZ  TIMER1,F
		GOTO    DEL_LOOP2
		DECFSZ  TIMER2,F
		GOTO    DEL_LOOP1
		return
;------------------------------------
dr2		call	d2
		call	d2
		call	d2
		return
;------------------------------------
dr2c		call	d2
		call	d2
		call	d2

		return
;------------------------------------

dr55		MOVLW   D'002'
		MOVWF   TIMER2
DEL_LOOPb1	MOVLW   D'100'
		MOVWF   TIMER1

DEL_LOOPb2	DECFSZ  TIMER1,F
		GOTO    DEL_LOOPb2
		DECFSZ  TIMER2,F
		GOTO    DEL_LOOPb1
		return

;-------------------------------------------------------------------------------
d55		movlw	D'255'
		movwf	TIMER2
d55_loop	decfsz	TIMER2,F
		goto	d55_loop
		return
;
dr55x		nop
		nop
		nop
		nop
		nop
		nop
		nop
		nop
		nop
		nop
		nop
		nop
		return
;
dr2n		call	d2
		call	d2
		return
;------------------------------------
dr77		call	d2
		return
;------------------------------------
;---------------------------------------------------------------------------

;xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx
;-----------------------------------------------------------------------------
;get a transmission from the keyboard
;-----------------------------------------------------------------------------
;xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx

get_datafk	nop
		btfsc	PORTB,D'000'
		goto	kbd_err01		;clock level is 1 instead of expected 0
		btfsc	PORTB,D'001'
		goto	kbd_err02		;data level is 1 instead of expected 0 (start bit)
;
get_datafk2	nop
		clrf	kbd_buffer

		call	get_bitfk
		btfsc	kbd_bb,D'001'
		bsf	kbd_buffer,D'000'

		call	get_bitfk
		btfsc	kbd_bb,D'001'
		bsf	kbd_buffer,D'001'

		call	get_bitfk
		btfsc	kbd_bb,D'001'
		bsf	kbd_buffer,D'002'

		call	get_bitfk
		btfsc	kbd_bb,D'001'
		bsf	kbd_buffer,D'003'

		call	get_bitfk
		btfsc	kbd_bb,D'001'
		bsf	kbd_buffer,D'004'

		call	get_bitfk
		btfsc	kbd_bb,D'001'
		bsf	kbd_buffer,D'005'

		call	get_bitfk
		btfsc	kbd_bb,D'001'
		bsf	kbd_buffer,D'006'

		call	get_bitfk
		btfsc	kbd_bb,D'001'
		bsf	kbd_buffer,D'007'

;get parity bit

		call	get_bitfk
		btfsc	kbd_bb,D'001'
		nop

;compare parity bit

		nop				;not now.... :-)

;wait for communication end - last bit should be "1"

		call	get_bitfk
		btfsc	kbd_bb,D'001'
		nop

		nop				;no check right now...

		return				;keyboard code is in kbd_buffer

;---------------------------------------------------------------------------------
;---------------------------------------------------------------------------------
;---------------------------------------------------------------------------------
write_char	nop
;send received data to PA
		movf	kbd_buffer,0
		movwf	bus_D
;		call	send_to_PA0
;and to the display
		movf	kbd_buffer,0
		call	convert2ascii

		call	d2

		return
;-----------------------------------------------------------------------------
;get bit from the keyboard during transmission
get_bitfk	nop
kbd_l2		btfss	PORTB,D'000'
		goto	kbd_l2			;wait to 1 on clock line
kbd_l3		btfsc	PORTB,D'000'
		goto	kbd_l3			;and now wait to 0 on clock
;
		movf	PORTB,0
		andlw	B'00000010'		;now we have the bit in W1
		movwf	kbd_bb

		return
;-----------------------------------------------------------------------------
kbd_err01	nop
		return
;-----------------------------
kbd_err02	nop
		return
;-----------------------------------------------------------------------------
;------------------------------------------------------------------------------------------------
;------------------------------------------------------------------------------------------------
;------------------------------------------------------------------------------------------------
;------------------------------------------------------------------------------
;-----------------------------------------------------------------------------

int_serv	nop
		nop
		nop

		bcf	PIR1, SSPIF
		PAGESEL	PAGE1
		call	wm_I
		PAGESEL	PAGE0

;restore values
		BANKSEL STATUS_TEMP	;Select regardless of current bank
		MOVF PCLATH_TEMP,W	;
		MOVWF PCLATH		;Restore PCLATH
		SWAPF STATUS_TEMP,W	;Swap STATUS_TEMP register into W
					;(sets bank to original state)
		MOVWF STATUS		;Move W into STATUS register
		SWAPF W_TEMP,F		;Swap W_TEMP
		SWAPF W_TEMP,W		;Swap W_TEMP into W


		bsf	INTCON,GIE
		bsf	INTCON,PEIE

		bsf	PIE1,SSPIE
		retfie

;XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
;xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx
;-----------------------------------------------------------------------------
;-----------------------------------------------------------------------------
; send data to the keyboard
;-----------------------------------------------------------------------------

send_ck		nop
		clrf	kbd_parity
		bsf	kbd_parity,D'000'

;-----------------------------------------------------------
;1.1. set RB4 (keaboard clock) to output ----------------------
		bsf	STATUS,RP0
		bcf	TRISB,D'000'
		bcf	STATUS,RP0
;-----------------------------------------------------------
;1.2. put KBD CLK to 0 ----------------------------------------
		bcf	PORTB,D'000'
		call	d57		;wait a while
;-----------------------------------------------------------
;2. put KBD DATA to 0 ---------------------------------------
		bsf	STATUS,RP0
		bcf	TRISB,D'001'
		bcf	STATUS,RP0

		bcf	PORTB,D'001'
;-----------------------------------------------------------
;3. release KBD CLK -----------------------------------------
		bsf	PORTB,D'000'

		bsf	STATUS,RP0
		bsf	TRISB,D'000'
		bcf	STATUS,RP0
;-----------------------------------------------------------
;4. wait for keayboard to bring the kbd clk low ------------
sck01		btfsc	PORTB,D'000'
		goto	sck01

		movf	kbd_out,0
		andlw	B'00000001'
		call	send_kbd_bit

		movf	kbd_out,0
		andlw	B'00000010'
		call	send_kbd_bit

		movf	kbd_out,0
		andlw	B'00000100'
		call	send_kbd_bit

		movf	kbd_out,0
		andlw	B'00001000'
		call	send_kbd_bit

		movf	kbd_out,0
		andlw	B'00010000'
		call	send_kbd_bit

		movf	kbd_out,0
		andlw	B'00100000'
		call	send_kbd_bit

		movf	kbd_out,0
		andlw	B'01000000'
		call	send_kbd_bit

		movf	kbd_out,0
		andlw	B'10000000'
		call	send_kbd_bit

;now send the parity bit...
		movf	kbd_parity,0
		andlw	B'00000001'
		call	send_kbd_bit

;		movlw	B'00000001'		;stop bit
;		andlw	B'00000001'
;		call	send_kbd_bit

		bsf	STATUS,RP0
		bsf	TRISB,D'001'		;data as input
		bcf	STATUS,RP0

w0079		btfsc	PORTB,D'000'
		goto	w0079

w0080		btfss	PORTB,D'000'
		goto	w0080

w0081		btfsc	PORTB,D'000'
		goto	w0081

w0082		btfss	PORTB,D'000'
		goto	w0082

		return

;-----------------------------------------------------------
send_kbd_bit	btfsc	STATUS,Z
		goto	send_kbd0
		goto	send_kbd1

send_kbd0	bcf	PORTB,D'001'

w0077		btfss	PORTB,D'000'
		goto	w0077

w0078		btfsc	PORTB,D'000'
		goto	w0078

		return

send_kbd1	bsf	PORTB,D'001'
		movlw	B'00000001'
		xorwf	kbd_parity,1
		goto	w0077
;-----------------------------------------------------------------------------
d57		MOVLW   D'030'
		MOVWF   TIMER2
DEL_LOOP5721	MOVLW   D'255'
		MOVWF   TIMER1
DEL_LOOP5722	DECFSZ  TIMER1,F
                GOTO    DEL_LOOP5722
                DECFSZ  TIMER2,F
                GOTO    DEL_LOOP5721
		return
;
dr01s		MOVLW   D'010'
		MOVWF   TIMER2
DEL_LOOP3_1s	MOVLW   D'255'
		MOVWF   TIMER1

DEL_LOOP3_2s	DECFSZ  TIMER1,F
                GOTO    DEL_LOOP3_2s
                DECFSZ  TIMER2,F
                GOTO    DEL_LOOP3_1s
		RETURN
;
;----------------------------------------------------------------------------
dr01		MOVLW   D'255'
		MOVWF   TIMER2
DEL_LOOP3_1	MOVLW   D'255'
		MOVWF   TIMER1

DEL_LOOP3_2	DECFSZ  TIMER1,F
                GOTO    DEL_LOOP3_2
                DECFSZ  TIMER2,F
                GOTO    DEL_LOOP3_1
		RETURN
;-------------------------------------


;XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
;XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
; display routines --- PAGE 1 --------------------------------------------------
;XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
;XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX


ORG 800
PAGE1
;xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx
;x                       LCD Display Communication Routines                                     x
;xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx

;RS  = RE2
;R/W = RE1
;E   = RE0

;D0 - D3 -- not connected

;D4  = RD0
;D5  = RD1
;D6  = RD2
;D7  = RD3


disp_init	nop
;1st step of init:
;		wait for LCD hardware reset (after power on) - no software reset performed...
		clrf	PORTD
		clrf	PORTE
		call	d2_p2

;-- SW Reset to be able to enter 4-bit mode
		bcf	PORTE,D'001'
		bcf	PORTE,D'002'
		bcf	PORTE,D'000'	;EN = 0 => disable


		movlw	B'00000011'
		movwf	PORTD
		bsf	PORTE,D'000'
		call	dy_comm
		bcf	PORTE,D'000'
		call	d2_p2

		movlw	B'00000011'
		movwf	PORTD
		bsf	PORTE,D'000'
		call	dy_comm
		bcf	PORTE,D'000'
		call	d2_p2

		movlw	B'00000011'
		movwf	PORTD
		bsf	PORTE,D'000'
		call	dy_comm
		bcf	PORTE,D'000'
		call	d2_p2

		movlw	B'00000010'
		movwf	PORTD
		bsf	PORTE,D'000'
		call	dy_comm
		bcf	PORTE,D'000'
		call	d2_p2

;-----------------------------------------------------------------
;2nd step of init:
;     command:           '001DNFxx' (SET FUNCTION)
;                            D = 1	--> 8 bit interface
;                            D = 0	--> 4 bit interface
;			 '001DNFxx'
;                             N = 1	--> 2 line display
;                             N = 0	--> 1 line display
;			 '001DNFxx'
;                              F = 1	--> 5x10
;			       F = 0	--> 5x7
;			 '001DNFxx'
		movlw	B'00101000'		;2 lines - (for disp 1x16: 1-8  = 00h - 0Fh
;                                                                          9-16 = 40h - 4Fh )
		call	send_comm
;-----------------------------------------------------------------
;3rd step of init:
;     command:           '00001DCB' (DISPLAY ON/OFF)
;                              D = 1	--> Display on
;			       D = 0	--> Display off
;           		 '00001DCB'
;                               C = 1	--> Cursor on
;				C = 0	--> Cursor off
;           		 '00001DCB'
;				 B = 1	--> Blinking on
;				 B = 0	--> Blinking off
		movlw	B'00001100'
		call	send_comm

;-----------------------------------------------------------------
;4th step of init:
;     command:           '000001DS' (ENTRY MODE SET)
;				D = 1	--> Increase
;				D = 0	--> Decrease
;		         '000001DS'
;				 S = 1	--> display is shifted
;				 S = 0	--> display is not shifted
;		         '000001DS'
		movlw	B'00000110'
		call	send_comm
;-----------------------------------------------------------------
;clear the display
		movlw	B'00000001'
		call	send_comm
		return
;------------------------------------------------------------------------------------------------
;8-bit data transfer routines
;------------------------------------------------------------------------------------------------
send_data	bcf	PORTE,D'000'		;EN -> 0
		movwf	data_tmp

		swapf	data_tmp,0
		andlw	B'00001111'
;;;;;		movwf	PORTD			;higher 4 bits are put to RA0-RA3
		call	set_PDD
		call	scbd			;send

		movf	data_tmp,0
		andlw	B'00001111'
;;;;;		movwf	PORTD			;lower 4 bits are put to RA0-RA3
		call	set_PDD
		call	scbd			;send

		return

;set control bits
scbd		bcf	PORTE,D'001'		;R/W -> 0 (write)
		bsf	PORTE,D'002'		;RS -> 1 (data)
		bsf	PORTE,D'000'		;EN -> 1
		call	dy_comm			;wait a while...
		bcf	PORTE,D'000'		;EN -> 0
		call	d1			;wait a little bit...

		return
;------------------------------------------------------------------------------------------------
send_comm	bcf	PORTE,D'000'		;EN -> 0
		movwf	data_tmp

		swapf	data_tmp,0
		andlw	B'00001111'
;;;;		movwf	PORTD			;higher 4 bits are put to RA0-RA3
		call	set_PDD
		call	scbc			;send

		movf	data_tmp,0
		andlw	B'00001111'
;;;;		movwf	PORTD			;lower 4 bits are put to RA0-RA3
		call	set_PDD
		call	scbc			;send

		return

;set control bits
scbc		bcf	PORTE,D'001'		;R/W -> 0 (write)
		bcf	PORTE,D'002'		;RS -> 0 (command)
		bsf	PORTE,D'000'		;EN -> 1
		call	dy_comm			;wait a while...
		bcf	PORTE,D'000'		;EN -> 0
		call	d1			;wait a little bit...
		return
;-----------------------------------------------------------------------------------------------
set_PDD		movwf	v_tmp1
		bsf	PORTD,D'000'
		btfss	v_tmp1,D'000'
		bcf	PORTD,D'000'

		bsf	PORTD,D'001'
		btfss	v_tmp1,D'001'
		bcf	PORTD,D'001'

		bsf	PORTD,D'002'
		btfss	v_tmp1,D'002'
		bcf	PORTD,D'002'

		bsf	PORTD,D'003'
		btfss	v_tmp1,D'003'
		bcf	PORTD,D'003'

		return
;----------------------------------------------
dy_comm		call	dy_comm2	;01
		call	dy_comm2	;02
		call	dy_comm2	;03
		return

		call	dy_comm2	;04
		call	dy_comm2	;05
		return

		call	dy_comm2	;06
		call	dy_comm2	;07
		call	dy_comm2	;08
		call	dy_comm2	;09
		call	dy_comm2	;10
		call	dy_comm2	;11
		call	dy_comm2	;12
		return


dy_comm2	MOVLW   D'010'			;delay value changed: original value was: 255
		MOVWF   TIMER2
DEL_LOOP22c	DECFSZ  TIMER2,F
                GOTO    DEL_LOOP22c
		return
;---------------------------------------------------------------------------
;
d1		call	d1b		;01
		call	d1b		;02
		return

		call	d1b		;03
		return

		call	d1b		;04
		call	d1b		;05
		call	d1b		;06
		call	d1b		;07
		call	d1b		;08
		call	d1b		;09
		return


d1b		MOVLW   D'255'
		MOVWF   TIMER2
DEL_LOOP11	DECFSZ  TIMER2,F
                GOTO    DEL_LOOP11
		return
;
d2_p2		MOVLW   D'255'
		MOVWF   TIMER2
DEL_LOOPx1	MOVLW   D'255'
		MOVWF   TIMER1

DEL_LOOPx2	DECFSZ  TIMER1,F
		GOTO    DEL_LOOPx2
		DECFSZ  TIMER2,F
		GOTO    DEL_LOOPx1
		return

;---------------------------------------------------------------------------
;---------------------------------------------------------------------------
;------------------------------------------------------------------------------------------------
clear_disp	call	disp_clr1
		call	disp_clr2
		return
;------------------------------------------------------------------------------------------------
;------------------------------------------------------------------------------------------------
n9_char		movlw	B'11000000'		;od znaku c.9 treba zmenit adresu na 40h
		call	send_comm

		movlw	B'10000000'
		movwf	disp_pos
		return
;------------------------------------------------------------------------------------------------
;------------------------------------------------------------------------------------------------
init2		movlw	B'00000001'
;		call	send_comm
		movlw	B'10000000'
		call	send_comm
		return

;xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx
;xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx
;
;character set
;
;xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx
;xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx

wm_space	movlw	' '
		call	send_data
		return

wm_dvojbodka	movlw	':'
		call	send_data
		return

wm_point	nop
wm_bodka	movlw	'.'
		call	send_data
		return

wm_otaznik	movlw	'?'
		call	send_data
		return

wm_sls		movlw	'/'
		call	send_data
		return

wm_kk		movlw	'#'
		call	send_data
		return

wm_minus	movlw	'-'
		call	send_data
		return

wm_plus		movlw	'+'
		call	send_data
		return

wm_hvizdicka	movlw	'*'
		call	send_data
		return

wm_zat1		movlw	'('
		call	send_data
		return

wm_zat2		movlw	')'
		call	send_data
		return

wm_sip1		movlw	'>'
		call	send_data
		return

wm_sip2		movlw	'<'
		call	send_data
		return

wm_stup		movlw	H'DF'
		call	send_data
		return

wm_rs		movlw	'='
		call	send_data
		return
;---------- cislice ----------------------------------------
wm_0		movlw	'0'
		call	send_data
		return

wm_1		movlw	'1'
		call	send_data
		return

wm_2		movlw	'2'
		call	send_data
		return

wm_3		movlw	'3'
		call	send_data
		return

wm_4		movlw	'4'
		call	send_data
		return

wm_5		movlw	'5'
		call	send_data
		return

wm_6		movlw	'6'
		call	send_data
		return

wm_7		movlw	'7'
		call	send_data
		return

wm_8		movlw	'8'
		call	send_data
		return

wm_9		movlw	'9'
		call	send_data
		return

;--------- velke pismena -----------------------------------
wm_A		movlw	'A'
		call	send_data
		return

wm_B		movlw	'B'
		call	send_data
		return

wm_C		movlw	'C'
		call	send_data
		return

wm_D		movlw	'D'
		call	send_data
		return

wm_E		movlw	'E'
		call	send_data
		return

wm_F		movlw	'F'
		call	send_data
		return

wm_G		movlw	'G'
		call	send_data
		return

wm_H		movlw	'H'
		call	send_data
		return

wm_I		movlw	'I'
		call	send_data
		return

wm_J		movlw	'J'
		call	send_data
		return

wm_K		movlw	'K'
		call	send_data
		return

wm_L		movlw	'L'
		call	send_data
		return

wm_M		movlw	'M'
		call	send_data
		return

wm_N		movlw	'N'
		call	send_data
		return

wm_O		movlw	'O'
		call	send_data
		return

wm_P		movlw	'P'
		call	send_data
		return

wm_Q		movlw	'Q'
		call	send_data
		return

wm_R		movlw	'R'
		call	send_data
		return

wm_S		movlw	'S'
		call	send_data
		return

wm_T		movlw	'T'
		call	send_data
		return

wm_U		movlw	'U'
		call	send_data
		return

wm_V		movlw	'V'
		call	send_data
		return

wm_W		movlw	'W'
		call	send_data
		return

wm_X		movlw	'X'
		call	send_data
		return

wm_Y		movlw	'Y'
		call	send_data
		return

wm_Z		movlw	'Z'
		call	send_data
		return

;--------- male pismena -----------------------------------

wm_a		movlw	'a'
		call	send_data
		return

wm_b		movlw	'b'
		call	send_data
		return

wm_c		movlw	'c'
		call	send_data
		return

wm_d		movlw	'd'
		call	send_data
		return

wm_e		movlw	'e'
		call	send_data
		return

wm_f		movlw	'f'
		call	send_data
		return

wm_g		movlw	'g'
		call	send_data
		return

wm_h		movlw	'h'
		call	send_data
		return

wm_i		movlw	'i'
		call	send_data
		return

wm_k		movlw	'k'
		call	send_data
		return

wm_l		movlw	'l'
		call	send_data
		return

wm_m		movlw	'm'
		call	send_data
		return

wm_n		movlw	'n'
		call	send_data
		return

wm_o		movlw	'o'
		call	send_data
		return

wm_p		movlw	'p'
		call	send_data
		return

wm_r		movlw	'r'
		call	send_data
		return

wm_s		movlw	's'
		call	send_data
		return

wm_t		movlw	't'
		call	send_data
		return

wm_u		movlw	'u'
		call	send_data
		return

wm_v		movlw	'v'
		call	send_data
		return

wm_w		movlw	'w'
		call	send_data
		return

wm_x		movlw	'x'
		call	send_data
		return

wm_y		movlw	'y'
		call	send_data
		return

wm_z		movlw	'z'
		call	send_data
		return
;xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx
;xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx
;xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx
;xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx

;-----------------------------------------------------------------------------
;-----------------------------------------------------------------------------
convert2ascii	movwf	c_tmp
;
		xorlw	H'15'
		btfsc	STATUS,Z
		goto	wm_Q
;
		movf	c_tmp,0
		xorlw	H'1D'
		btfsc	STATUS,Z
		goto	wm_W
;
		movf	c_tmp,0
		xorlw	H'24'
		btfsc	STATUS,Z
		goto	wm_E
;
		movf	c_tmp,0
		xorlw	H'2D'
		btfsc	STATUS,Z
		goto	wm_R
;
		movf	c_tmp,0
		xorlw	H'2C'
		btfsc	STATUS,Z
		goto	wm_T
;
		movf	c_tmp,0
		xorlw	H'35'
		btfsc	STATUS,Z
		goto	wm_Y
;
		movf	c_tmp,0
		xorlw	H'3C'
		btfsc	STATUS,Z
		goto	wm_U
;
		movf	c_tmp,0
		xorlw	H'43'
		btfsc	STATUS,Z
		goto	wm_I
;
		movf	c_tmp,0
		xorlw	H'44'
		btfsc	STATUS,Z
		goto	wm_O
;
		movf	c_tmp,0
		xorlw	H'4D'
		btfsc	STATUS,Z
		goto	wm_P
;
		movf	c_tmp,0
		xorlw	H'16'
		btfsc	STATUS,Z
		goto	wm_1
;
		movf	c_tmp,0
		xorlw	H'1E'
		btfsc	STATUS,Z
		goto	wm_2
;
		movf	c_tmp,0
		xorlw	H'26'
		btfsc	STATUS,Z
		goto	wm_3
;
		movf	c_tmp,0
		xorlw	H'25'
		btfsc	STATUS,Z
		goto	wm_4
;
		movf	c_tmp,0
		xorlw	H'2E'
		btfsc	STATUS,Z
		goto	wm_5
;
		movf	c_tmp,0
		xorlw	H'36'
		btfsc	STATUS,Z
		goto	wm_6
;
		movf	c_tmp,0
		xorlw	H'3D'
		btfsc	STATUS,Z
		goto	wm_7
;
		movf	c_tmp,0
		xorlw	H'3E'
		btfsc	STATUS,Z
		goto	wm_8
;
		movf	c_tmp,0
		xorlw	H'46'
		btfsc	STATUS,Z
		goto	wm_9
;
		movf	c_tmp,0
		xorlw	H'45'
		btfsc	STATUS,Z
		goto	wm_0
;
		movf	c_tmp,0
		xorlw	H'1C'
		btfsc	STATUS,Z
		goto	wm_A
;
		movf	c_tmp,0
		xorlw	H'1B'
		btfsc	STATUS,Z
		goto	wm_S
;
		movf	c_tmp,0
		xorlw	H'23'
		btfsc	STATUS,Z
		goto	wm_D
;
		movf	c_tmp,0
		xorlw	H'2B'
		btfsc	STATUS,Z
		goto	wm_F
;
		movf	c_tmp,0
		xorlw	H'34'
		btfsc	STATUS,Z
		goto	wm_G
;
		movf	c_tmp,0
		xorlw	H'33'
		btfsc	STATUS,Z
		goto	wm_H
;
		movf	c_tmp,0
		xorlw	H'3B'
		btfsc	STATUS,Z
		goto	wm_J
;
		movf	c_tmp,0
		xorlw	H'42'
		btfsc	STATUS,Z
		goto	wm_K
;
		movf	c_tmp,0
		xorlw	H'4B'
		btfsc	STATUS,Z
		goto	wm_L
;
		movf	c_tmp,0
		xorlw	H'1A'
		btfsc	STATUS,Z
		goto	wm_Z
;
		movf	c_tmp,0
		xorlw	H'22'
		btfsc	STATUS,Z
		goto	wm_X
;
		movf	c_tmp,0
		xorlw	H'21'
		btfsc	STATUS,Z
		goto	wm_C
;
		movf	c_tmp,0
		xorlw	H'2A'
		btfsc	STATUS,Z
		goto	wm_V
;
		movf	c_tmp,0
		xorlw	H'32'
		btfsc	STATUS,Z
		goto	wm_B
;
		movf	c_tmp,0
		xorlw	H'31'
		btfsc	STATUS,Z
		goto	wm_N
;
		movf	c_tmp,0
		xorlw	H'3A'
		btfsc	STATUS,Z
		goto	wm_M
;
		movf	c_tmp,0
		xorlw	H'29'
		btfsc	STATUS,Z
		goto	wm_space
;
		movf	c_tmp,0
		xorlw	H'6C'
		btfsc	STATUS,Z
		goto	wm_7
;
		movf	c_tmp,0
		xorlw	H'75'
		btfsc	STATUS,Z
		goto	wm_8
;
		movf	c_tmp,0
		xorlw	H'7D'
		btfsc	STATUS,Z
		goto	wm_9
;
		movf	c_tmp,0
		xorlw	H'6B'
		btfsc	STATUS,Z
		goto	wm_4
;
		movf	c_tmp,0
		xorlw	H'73'
		btfsc	STATUS,Z
		goto	wm_5
;
		movf	c_tmp,0
		xorlw	H'74'
		btfsc	STATUS,Z
		goto	wm_6
;
		movf	c_tmp,0
		xorlw	H'69'
		btfsc	STATUS,Z
		goto	wm_1
;
		movf	c_tmp,0
		xorlw	H'72'
		btfsc	STATUS,Z
		goto	wm_2
;
		movf	c_tmp,0
		xorlw	H'7A'
		btfsc	STATUS,Z
		goto	wm_3
;
		movf	c_tmp,0
		xorlw	H'70'
		btfsc	STATUS,Z
		goto	wm_0
;
		movf	c_tmp,0
		xorlw	H'71'
		btfsc	STATUS,Z
		goto	wm_bodka
;
		movf	c_tmp,0
		xorlw	H'79'
		btfsc	STATUS,Z
		goto	wm_plus
;
		movf	c_tmp,0
		xorlw	H'7B'
		btfsc	STATUS,Z
		goto	wm_minus
;
		movf	c_tmp,0
		xorlw	H'7C'
		btfsc	STATUS,Z
		goto	wm_hvizdicka

		movf	c_tmp,0
		xorlw	H'76'			;ESC
		btfsc	STATUS,Z
;!!!!!!		goto	main_s


;not known character - either transmission error or not decoded... - do nothing...
		decf	disp_pos,1
		
		return


		return
;-----------------------------------------------------------------------------
;---------------------------------------------------------------------------------------------
;XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
;XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
;-------------------------------------------------------------------------------
write_hex	nop

		movf	kbd_buffer,0
		andlw	B'11110000'
		xorlw	H'00'
		btfsc	STATUS,Z
		call	wm_0

		movf	kbd_buffer,0
		andlw	B'11110000'
		xorlw	H'10'
		btfsc	STATUS,Z
		call	wm_1

		movf	kbd_buffer,0
		andlw	B'11110000'
		xorlw	H'20'
		btfsc	STATUS,Z
		call	wm_2

		movf	kbd_buffer,0
		andlw	B'11110000'
		xorlw	H'30'
		btfsc	STATUS,Z
		call	wm_3

		movf	kbd_buffer,0
		andlw	B'11110000'
		xorlw	H'40'
		btfsc	STATUS,Z
		call	wm_4

		movf	kbd_buffer,0
		andlw	B'11110000'
		xorlw	H'50'
		btfsc	STATUS,Z
		call	wm_5

		movf	kbd_buffer,0
		andlw	B'11110000'
		xorlw	H'60'
		btfsc	STATUS,Z
		call	wm_6

		movf	kbd_buffer,0
		andlw	B'11110000'
		xorlw	H'70'
		btfsc	STATUS,Z
		call	wm_7

		movf	kbd_buffer,0
		andlw	B'11110000'
		xorlw	H'80'
		btfsc	STATUS,Z
		call	wm_8

		movf	kbd_buffer,0
		andlw	B'11110000'
		xorlw	H'90'
		btfsc	STATUS,Z
		call	wm_9

		movf	kbd_buffer,0
		andlw	B'11110000'
		xorlw	H'A0'
		btfsc	STATUS,Z
		call	wm_A

		movf	kbd_buffer,0
		andlw	B'11110000'
		xorlw	H'B0'
		btfsc	STATUS,Z
		call	wm_B

		movf	kbd_buffer,0
		andlw	B'11110000'
		xorlw	H'C0'
		btfsc	STATUS,Z
		call	wm_C

		movf	kbd_buffer,0
		andlw	B'11110000'
		xorlw	H'D0'
		btfsc	STATUS,Z
		call	wm_D

		movf	kbd_buffer,0
		andlw	B'11110000'
		xorlw	H'E0'
		btfsc	STATUS,Z
		call	wm_E

		movf	kbd_buffer,0
		andlw	B'11110000'
		xorlw	H'F0'
		btfsc	STATUS,Z
		call	wm_F
;
		movf	kbd_buffer,0
		andlw	B'00001111'
		xorlw	H'00'
		btfsc	STATUS,Z
		goto	wm_0

		movf	kbd_buffer,0
		andlw	B'00001111'
		xorlw	H'01'
		btfsc	STATUS,Z
		goto	wm_1

		movf	kbd_buffer,0
		andlw	B'00001111'
		xorlw	H'02'
		btfsc	STATUS,Z
		goto	wm_2

		movf	kbd_buffer,0
		andlw	B'00001111'
		xorlw	H'03'
		btfsc	STATUS,Z
		goto	wm_3

		movf	kbd_buffer,0
		andlw	B'00001111'
		xorlw	H'04'
		btfsc	STATUS,Z
		goto	wm_4

		movf	kbd_buffer,0
		andlw	B'00001111'
		xorlw	H'05'
		btfsc	STATUS,Z
		goto	wm_5

		movf	kbd_buffer,0
		andlw	B'00001111'
		xorlw	H'06'
		btfsc	STATUS,Z
		goto	wm_6

		movf	kbd_buffer,0
		andlw	B'00001111'
		xorlw	H'07'
		btfsc	STATUS,Z
		goto	wm_7

		movf	kbd_buffer,0
		andlw	B'00001111'
		xorlw	H'08'
		btfsc	STATUS,Z
		goto	wm_8

		movf	kbd_buffer,0
		andlw	B'00001111'
		xorlw	H'09'
		btfsc	STATUS,Z
		goto	wm_9

		movf	kbd_buffer,0
		andlw	B'00001111'
		xorlw	H'0A'
		btfsc	STATUS,Z
		goto	wm_A

		movf	kbd_buffer,0
		andlw	B'00001111'
		xorlw	H'0B'
		btfsc	STATUS,Z
		goto	wm_B

		movf	kbd_buffer,0
		andlw	B'00001111'
		xorlw	H'0C'
		btfsc	STATUS,Z
		goto	wm_C

		movf	kbd_buffer,0
		andlw	B'00001111'
		xorlw	H'0D'
		btfsc	STATUS,Z
		goto	wm_D

		movf	kbd_buffer,0
		andlw	B'00001111'
		xorlw	H'0E'
		btfsc	STATUS,Z
		goto	wm_E

		movf	kbd_buffer,0
		andlw	B'00001111'
		xorlw	H'0F'
		btfsc	STATUS,Z
		goto	wm_F


		return
;---------------------------------------------------------------------------------


;xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx
;xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx
;
; write texts to the display
;
;xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx
;xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx
;---------------------------------------------------------------------------
w_pcsl		movlw	B'11001100'		;line 2 / char12
		call	send_comm

		call	wm_space
		call	wm_space
		call	wm_space
		call	wm_space

		return
;---------------------------------------------------------------------------
w_otaznik	call	send_comm
;
		call	wm_otaznik
;
		movf	tmp_ot,0
		call	send_comm
		return
;---------------------------------------------------------------------------
w_scr5		movlw	B'10001100'		;line 1 / char12
		call	send_comm


		return
;---------------------------------------------------------------------------
;--------------------------------------------------------------------------
w_kbd		movlw	B'11001100'		;line 1 / char12
		call	send_comm

		movlw	'0'
		btfsc	PORTB,D'000'
		movlw	'1'
		call	send_data

		movlw	'0'
		btfsc	PORTB,D'001'
		movlw	'1'
		call	send_data

		return
;--------------------------------------------------------------------------
w_data		movlw	B'11000000'		;line 2 / char0
		call	send_comm


		return
;--------------------------------------------------------------------------
w_search	movlw	B'10001100'		;line 1 / char12
		call	send_comm
		return
;
;------------------------------------------------------------------------------
;-----------------------------------------------------------------------------
w_search_again	movlw	B'11000000'		;line 2 / char0
		call	send_comm

		call	wm_R
		return
;---------------------------------------------------------------------------
disp_clr1	movlw	B'10000000'		;line 1 / char0
		call	send_comm

		call	wm_space	;01
		call	wm_space	;02
		call	wm_space	;03
		call	wm_space	;04
		call	wm_space	;05
		call	wm_space	;06
		call	wm_space	;07
		call	wm_space	;08
		call	wm_space	;09
		call	wm_space	;10
		call	wm_space	;11
		call	wm_space	;12
		call	wm_space	;13
		call	wm_space	;14
		call	wm_space	;15
		call	wm_space	;16
		call	wm_space	;17
		call	wm_space	;18
		call	wm_space	;19
		call	wm_space	;20
		call	wm_space	;21
		call	wm_space	;22
		call	wm_space	;23
		call	wm_space	;24

		return
;---------------------------------------------------------------------------
disp_clr2	movlw	B'11000000'		;line 2 / char0
		call	send_comm

		call	wm_space	;01
		call	wm_space	;02
		call	wm_space	;03
		call	wm_space	;04
		call	wm_space	;05
		call	wm_space	;06
		call	wm_space	;07
		call	wm_space	;08
		call	wm_space	;09
		call	wm_space	;10
		call	wm_space	;11
		call	wm_space	;12
		call	wm_space	;13
		call	wm_space	;14
		call	wm_space	;15
		call	wm_space	;16
		call	wm_space	;17
		call	wm_space	;18
		call	wm_space	;19
		call	wm_space	;20
		call	wm_space	;21
		call	wm_space	;22
		call	wm_space	;23
		call	wm_space	;24

		return

;-------------------------------------------------------------------------------
w_release	call	init2

		movlw	B'10000000'		;row: 1 / col: 0
		call	send_comm

		call	wm_A
		call	wm_t
		call	wm_m
		call	wm_e
		call	wm_l
		call	wm_space
		call	wm_I
		call	wm_2
		call	wm_C
		call	wm_space
		call	wm_P
		call	wm_r
		call	wm_o
		call	wm_g
		call	wm_r
		call	wm_a
		call	wm_m
		call	wm_m
		call	wm_e
		call	wm_r
		call	wm_space
		call	wm_space
		call	wm_space

		movlw	B'11000000'		;row: 2 / col: 0
		call	send_comm

		call	wm_v
		call	wm_1
		call	wm_point
		call	wm_3
		call	wm_1
		call	wm_space
		call	wm_minus
		call	wm_space
		call	wm_0
		call	wm_8
		call	wm_point
		call	wm_0
		call	wm_8
		call	wm_point
		call	wm_2
		call	wm_0
		call	wm_1
		call	wm_3

		return
;------------------------------------------------------------------------------------------
w_L2a		call	wm_8
		call	wm_9
;		goto	back1
;------------------------------------------------------------------------------------------
w_mode		movlw	B'11001111'		;row: 2 / col: 15
		call	send_comm
		return
;------------------------------------------------------------------------------------------
;
w_L2b		call	wm_L
		call	wm_2
		return
;------------------------------------------------------------------------------
w_wkbd		movlw	B'10000000'		;row: 1 / col: 0
		call	send_comm

		call	wm_w
		call	wm_a
		call	wm_i
		call	wm_t
		call	wm_i
		call	wm_n
		call	wm_g
		call	wm_space
		call	wm_f
		call	wm_o
		call	wm_r
		call	wm_space
		call	wm_k
		call	wm_e
		call	wm_y
		call	wm_b
		call	wm_o
		call	wm_a
		call	wm_r
		call	wm_d
		call	wm_point
		call	wm_point
		call	wm_point

		return
;-------------------------------------------------------------------------------
;-------------------------------------------------------------------------------
;-------------------------------------------------------------------------------
w_eeprom_data	call	send_comm

;first write the address (2 bytes)
		movf	v_ad1,0
		movwf	kbd_buffer
		call	write_hex
;
		movf	v_ad0,0
		movwf	kbd_buffer
		call	write_hex
;
		call	wm_dvojbodka
;
		movf	v_d0,0
		movwf	kbd_buffer
		call	write_hex

		call	wm_space

		movf	v_d1,0
		movwf	kbd_buffer
		call	write_hex

		call	wm_space

		movf	v_d2,0
		movwf	kbd_buffer
		call	write_hex

		call	wm_space

		movf	v_d3,0
		movwf	kbd_buffer
		call	write_hex

		call	wm_space

		movf	v_d4,0
		movwf	kbd_buffer
		call	write_hex

		call	wm_space

		movf	v_d5,0
		movwf	kbd_buffer
		call	write_hex

		call	wm_space
;
		return
;-------------------------------------------------------------------------------
;-------------------------------------------------------------------------------
w_pickit2	movlw	B'10000000'		;row: 1 / col: 0
		call	send_comm

		call	wm_P
		call	wm_I
		call	wm_C
		call	wm_k
		call	wm_i
		call	wm_t
		call	wm_2
		call	wm_space
		call	wm_e
		call	wm_n
		call	wm_a
		call	wm_b
		call	wm_l
		call	wm_e
		call	wm_d
		call	wm_bodka
		call	wm_bodka
		call	wm_bodka

		movlw	B'11000000'		;row: 2 / col: 0
		call	send_comm

		call	wm_E
		call	wm_S
		call	wm_C
		call	wm_space
		call	wm_t
		call	wm_o
		call	wm_space
		call	wm_r
		call	wm_e
		call	wm_t
		call	wm_u
		call	wm_r
		call	wm_n
		return
;-----------------------------------------------------------------
;-----------------------------------------------------------------
w_m001		movlw	B'10000000'		;row: 1 / col: 0
		call	send_comm

		call	wm_zat1
		call	wm_P
		call	wm_zat2
		call	wm_I
		call	wm_C
		call	wm_k
		call	wm_i
		call	wm_t
		call	wm_2
		call	wm_space
		call	wm_c
		call	wm_o
		call	wm_n
		call	wm_n
		call	wm_e
		call	wm_c
		call	wm_t

		movlw	B'11000000'		;row: 2 / col: 0
		call	send_comm

		call	wm_zat1
		call	wm_A
		call	wm_zat2
		call	wm_t
		call	wm_m
		call	wm_e
		call	wm_l
		call	wm_space
		call	wm_p
		call	wm_r
		call	wm_o
		call	wm_g
		call	wm_sls
		call	wm_zat1
		call	wm_E
		call	wm_zat2
		call	wm_E
		call	wm_P
		call	wm_R
		call	wm_O
		call	wm_M
		return
;-----------------------------------------------------------------
;-----------------------------------------------------------------
w_att001	movlw	B'10000000'		;row: 1 / col: 0
		call	send_comm

		movf	v_att_chip,0
		xorlw	H'FF'
		btfsc	STATUS,Z
		goto	w_att_nc2

		movf	v_att_chip,0
		xorlw	H'FE'
		btfsc	STATUS,Z
		goto	w_att_nc1

		movf	v_att_chip,0
		xorlw	H'01'
		btfsc	STATUS,Z
		goto	w_att_2313

		movf	v_att_chip,0
		xorlw	H'02'
		btfsc	STATUS,Z
		goto	w_att_4313

		movf	v_att_chip,0
		xorlw	H'03'
		btfsc	STATUS,Z
		goto	w_att_88

		movf	v_att_chip,0
		xorlw	H'04'
		btfsc	STATUS,Z
		goto	w_att_461A

		movf	v_att_chip,0
		xorlw	H'05'
		btfsc	STATUS,Z
		goto	w_att_48

		movf	v_att_chip,0
		xorlw	H'06'
		btfsc	STATUS,Z
		goto	w_att_261A

		movf	v_att_chip,0
		xorlw	H'07'
		btfsc	STATUS,Z
		goto	w_att_861A

		movf	v_att_chip,0
		xorlw	H'08'
		btfsc	STATUS,Z
		goto	w_att_26

		movf	v_att_chip,0
		xorlw	H'11'
		btfsc	STATUS,Z
		goto	w_att_m48A

		movf	v_att_chip,0
		xorlw	H'12'
		btfsc	STATUS,Z
		goto	w_att_m48PA

		movf	v_att_chip,0
		xorlw	H'13'
		btfsc	STATUS,Z
		goto	w_att_m88A

		movf	v_att_chip,0
		xorlw	H'14'
		btfsc	STATUS,Z
		goto	w_att_m88PA

		movf	v_att_chip,0
		xorlw	H'15'
		btfsc	STATUS,Z
		goto	w_att_m168A

		movf	v_att_chip,0
		xorlw	H'16'
		btfsc	STATUS,Z
		goto	w_att_m168PA

		movf	v_att_chip,0
		xorlw	H'17'
		btfsc	STATUS,Z
		goto	w_att_m328

		movf	v_att_chip,0
		xorlw	H'18'
		btfsc	STATUS,Z
		goto	w_att_m328P

		call	wm_E
		call	wm_r
		call	wm_r
		call	wm_o
		call	wm_r
		return

w_att001b	movlw	B'11000000'		;row: 2 / col: 0
		call	send_comm

		call	wm_R
		call	wm_d
		call	wm_sls
		call	wm_E
		call	wm_r
		call	wm_a
		call	wm_s
		call	wm_e
		call	wm_sls
		call	wm_W
		call	wm_r
		call	wm_i
		call	wm_t
		call	wm_e
		call	wm_sls
		call	wm_F
		call	wm_L
		call	wm_sls
		call	wm_V
		call	wm_e
		call	wm_r
		call	wm_i
		call	wm_f
		call	wm_y

		return
;------------------------------------------------------------------------------
w_att_2313	call	wm_C
		call	wm_h
		call	wm_i
		call	wm_p
		call	wm_dvojbodka
		call	wm_space
		call	wm_A
		call	wm_T
		call	wm_t
		call	wm_i
		call	wm_n
		call	wm_y
		call	wm_2
		call	wm_3
		call	wm_1
		call	wm_3
		call	wm_zat1
		call	wm_A
		call	wm_zat2
		goto	w_att001b

w_att_4313	call	wm_C
		call	wm_h
		call	wm_i
		call	wm_p
		call	wm_dvojbodka
		call	wm_space
		call	wm_A
		call	wm_T
		call	wm_t
		call	wm_i
		call	wm_n
		call	wm_y
		call	wm_4
		call	wm_3
		call	wm_1
		call	wm_3
		goto	w_att001b

w_att_88	call	wm_C
		call	wm_h
		call	wm_i
		call	wm_p
		call	wm_dvojbodka
		call	wm_space
		call	wm_A
		call	wm_T
		call	wm_t
		call	wm_i
		call	wm_n
		call	wm_y
		call	wm_8
		call	wm_8
		goto	w_att001b

w_att_48	call	wm_C
		call	wm_h
		call	wm_i
		call	wm_p
		call	wm_dvojbodka
		call	wm_space
		call	wm_A
		call	wm_T
		call	wm_t
		call	wm_i
		call	wm_n
		call	wm_y
		call	wm_4
		call	wm_8
		goto	w_att001b

w_att_461A	call	wm_C
		call	wm_h
		call	wm_i
		call	wm_p
		call	wm_dvojbodka
		call	wm_space
		call	wm_A
		call	wm_T
		call	wm_t
		call	wm_i
		call	wm_n
		call	wm_y
		call	wm_4
		call	wm_6
		call	wm_1
		call	wm_A
		goto	w_att001b

w_att_261A	call	wm_C
		call	wm_h
		call	wm_i
		call	wm_p
		call	wm_dvojbodka
		call	wm_space
		call	wm_A
		call	wm_T
		call	wm_t
		call	wm_i
		call	wm_n
		call	wm_y
		call	wm_2
		call	wm_6
		call	wm_1
		call	wm_A
		goto	w_att001b

w_att_861A	call	wm_C
		call	wm_h
		call	wm_i
		call	wm_p
		call	wm_dvojbodka
		call	wm_space
		call	wm_A
		call	wm_T
		call	wm_t
		call	wm_i
		call	wm_n
		call	wm_y
		call	wm_8
		call	wm_6
		call	wm_1
		call	wm_A
		goto	w_att001b

w_att_26	call	wm_C
		call	wm_h
		call	wm_i
		call	wm_p
		call	wm_dvojbodka
		call	wm_space
		call	wm_A
		call	wm_T
		call	wm_t
		call	wm_i
		call	wm_n
		call	wm_y
		call	wm_2
		call	wm_6
		goto	w_att001b

w_att_m48A	call	wm_C
		call	wm_h
		call	wm_i
		call	wm_p
		call	wm_dvojbodka
		call	wm_space
		call	wm_A
		call	wm_T
		call	wm_m
		call	wm_e
		call	wm_g
		call	wm_a
		call	wm_4
		call	wm_8
		call	wm_A
		goto	w_att001b

w_att_m48PA	call	wm_C
		call	wm_h
		call	wm_i
		call	wm_p
		call	wm_dvojbodka
		call	wm_space
		call	wm_A
		call	wm_T
		call	wm_m
		call	wm_e
		call	wm_g
		call	wm_a
		call	wm_4
		call	wm_8
		call	wm_P
		call	wm_A
		goto	w_att001b


w_att_m88A	call	wm_C
		call	wm_h
		call	wm_i
		call	wm_p
		call	wm_dvojbodka
		call	wm_space
		call	wm_A
		call	wm_T
		call	wm_m
		call	wm_e
		call	wm_g
		call	wm_a
		call	wm_8
		call	wm_8
		call	wm_A
		goto	w_att001b

w_att_m88PA	call	wm_C
		call	wm_h
		call	wm_i
		call	wm_p
		call	wm_dvojbodka
		call	wm_space
		call	wm_A
		call	wm_T
		call	wm_m
		call	wm_e
		call	wm_g
		call	wm_a
		call	wm_8
		call	wm_8
		call	wm_P
		call	wm_A
		goto	w_att001b

w_att_m168A	call	wm_C
		call	wm_h
		call	wm_i
		call	wm_p
		call	wm_dvojbodka
		call	wm_space
		call	wm_A
		call	wm_T
		call	wm_m
		call	wm_e
		call	wm_g
		call	wm_a
		call	wm_1
		call	wm_6
		call	wm_8
		call	wm_A
		goto	w_att001b

w_att_m168PA	call	wm_C
		call	wm_h
		call	wm_i
		call	wm_p
		call	wm_dvojbodka
		call	wm_space
		call	wm_A
		call	wm_T
		call	wm_m
		call	wm_e
		call	wm_g
		call	wm_a
		call	wm_1
		call	wm_6
		call	wm_8
		call	wm_P
		call	wm_A
		goto	w_att001b

w_att_m328	call	wm_C
		call	wm_h
		call	wm_i
		call	wm_p
		call	wm_dvojbodka
		call	wm_space
		call	wm_A
		call	wm_T
		call	wm_m
		call	wm_e
		call	wm_g
		call	wm_a
		call	wm_3
		call	wm_2
		call	wm_8
		goto	w_att001b

w_att_m328P	call	wm_C
		call	wm_h
		call	wm_i
		call	wm_p
		call	wm_dvojbodka
		call	wm_space
		call	wm_A
		call	wm_T
		call	wm_m
		call	wm_e
		call	wm_g
		call	wm_a
		call	wm_3
		call	wm_2
		call	wm_8
		call	wm_P
		goto	w_att001b

;---------------------------------------------------------------------------------


w_att_nc1	call	wm_U
		call	wm_n
		call	wm_k
		call	wm_n
		call	wm_o
		call	wm_w
		call	wm_n
		call	wm_space
		call	wm_d
		call	wm_e
		call	wm_v
		call	wm_i
		call	wm_c
		call	wm_e

		goto	w_att_wsb

;----------------------------------------------
w_att_nc2	call	wm_N
		call	wm_o
		call	wm_space
		call	wm_d
		call	wm_e
		call	wm_v
		call	wm_i
		call	wm_c
		call	wm_e

w_att_wsb	movlw	B'11000000'
		call	send_comm
;
		call	wm_S
		call	wm_i
		call	wm_g
		call	wm_n
		call	wm_a
		call	wm_t
		call	wm_u
		call	wm_r
		call	wm_e
		call	wm_dvojbodka
		call	wm_space

		movf	v_att_s0,0
		movwf	kbd_buffer
		call	write_hex
		call	wm_minus

		movf	v_att_s1,0
		movwf	kbd_buffer
		call	write_hex
		call	wm_minus

		movf	v_att_s2,0
		movwf	kbd_buffer
		call	write_hex

		return
;-----------------------------------------------------------------
;write ATtiny signature to 2nd line
;
w_att_sgn	movlw	B'10000000'		;row: 1 / col: 0
		call	send_comm

		call	wm_S
		call	wm_i
		call	wm_g
		call	wm_n
		call	wm_a
		call	wm_t
		call	wm_u
		call	wm_r
		call	wm_e
		call	wm_space
		call	wm_space
		call	wm_space
		call	wm_space
		call	wm_space
		call	wm_space
		call	wm_space
		call	wm_space
		call	wm_space
		call	wm_space
		call	wm_space
		call	wm_space
		call	wm_space

		movlw	B'11000000'		;row: 2 / col: 0
		call	send_comm

		call	wm_S
		call	wm_0
		call	wm_dvojbodka
		movf	v_att_s0,0		;S0
		movwf	kbd_buffer
		call	write_hex

		call	wm_space

		call	wm_S
		call	wm_1
		call	wm_dvojbodka
		movf	v_att_s1,0		;S1
		movwf	kbd_buffer
		call	write_hex

		call	wm_space

		call	wm_S
		call	wm_2
		call	wm_dvojbodka
		movf	v_att_s2,0		;S2
		movwf	kbd_buffer
		call	write_hex

		return
;-----------------------------------------------------------------------
w_input01	movlw	B'11000000'		;row: 2 / col: 0
		call	send_comm

		call	wm_I
		call	wm_1
		call	wm_dvojbodka
		movf	v_byte,0
		movwf	kbd_buffer
		call	write_hex
		call	wm_H

		return
;-----------------------------------------------------------------------
;-----------------------------------------------------------------------
w_att_flash01	movlw	B'10000000'		;row: 1 / col: 0
		call	send_comm
;
		call	wm_0
		call	wm_0
		call	wm_0
		call	wm_dvojbodka
		return
;-----------------------------------------------------------------------
w_at01m_prog	movlw	B'10000000'		;row: 1 / col: 0
		call	send_comm

		call	wm_W
		call	wm_r
		call	wm_i
		call	wm_t
		call	wm_i
		call	wm_n
		call	wm_g
		call	wm_space
		call	wm_P
		call	wm_a
		call	wm_g
		call	wm_e

w_at01m_g1	movlw	B'10010000'		;row: 1 / col: 16
		call	send_comm

		call	wm_S
		call	wm_i
		call	wm_z
		call	wm_e
		call	wm_dvojbodka
		call	wm_space
		movf	v_page_size,0
		movwf	kbd_buffer
		call	write_hex

		movlw	B'11000000'		;row: 2 / col: 0
		call	send_comm

		call	wm_R
		call	wm_e
		call	wm_m
		call	wm_a
		call	wm_i
		call	wm_n
		call	wm_i
		call	wm_n
		call	wm_g

		return
;-----------------------------------------------------------------------
w_at01_prog	movlw	B'10001101'		;row: 1 / col: 13
		call	send_comm

		movf	v_att_pgs_w,0
		movwf	kbd_buffer
		call	write_hex

		movlw	B'11001101'		;row: 2 / col: 13
		call	send_comm

		movf	v_att_pgs,0
		movwf	kbd_buffer
		call	write_hex
		return
;-----------------------------------------------------------------------
w_att013_1	movlw	B'10000000'		;row: 1 / col: 0
		call	send_comm

		call	wm_E
		call	wm_x
		call	wm_t
		call	wm_e
		call	wm_n
		call	wm_d
		call	wm_e
		call	wm_d
		call	wm_dvojbodka
		call	wm_space
		call	wm_space
		call	wm_space
		call	wm_S
		call	wm_E
		call	wm_L
		call	wm_F
		call	wm_P
		call	wm_R
		call	wm_G
		call	wm_E
		call	wm_N
		call	wm_dvojbodka
		movlw	'0'
		btfsc	v_fuse_e,D'000'
		movlw	'1'
		call	send_data

		call	w_fuse_mod_flag

		return
;------------------------------------------------------------------------------
;------------------------------------------------------------------------------
w_fuse_mod_flag	movlw	B'11000000'		;row: 2 / col: 0
		call	send_comm
		movlw	' '
		btfsc	v_fuse_mod,D'000'
		movlw	'M'
		call	send_data

		return
;------------------------------------------------------------------------------
;------------------------------------------------------------------------------
w_att013_2	movlw	B'10000000'		;row: 1 / col: 0
		call	send_comm

		call	wm_H
		call	wm_i
		call	wm_g
		call	wm_h
		call	wm_1
		call	wm_dvojbodka
		call	wm_space
		call	wm_D
		call	wm_W
		call	wm_E
		call	wm_N
		call	wm_space
		call	wm_dvojbodka
		movlw	'0'
		btfsc	v_fuse_h,D'007'
		movlw	'1'
		call	send_data
		call	wm_space
		call	wm_space
		call	wm_E
		call	wm_E
		call	wm_S
		call	wm_A
		call	wm_V
		call	wm_E
		call	wm_dvojbodka
		movlw	'0'
		btfsc	v_fuse_h,D'006'
		movlw	'1'
		call	send_data

		call	w_fuse_mod_flag

		movlw	B'11000111'		;row: 2 / col: 7
		call	send_comm
		call	wm_S
		call	wm_P
		call	wm_I
		call	wm_E
		call	wm_N
		call	wm_dvojbodka
		movlw	'0'
		btfsc	v_fuse_h,D'005'
		movlw	'1'
		call	send_data
		call	wm_space
		call	wm_space
		call	wm_space
		call	wm_W
		call	wm_D
		call	wm_T
		call	wm_O
		call	wm_N
		call	wm_dvojbodka
		movlw	'0'
		btfsc	v_fuse_h,D'004'
		movlw	'1'
		call	send_data

		return
;------------------------------------------------------------------------------
;------------------------------------------------------------------------------
w_att013_3	movlw	B'10000000'		;row: 1 / col: 0
		call	send_comm

		call	wm_H
		call	wm_i
		call	wm_g
		call	wm_h
		call	wm_2
		call	wm_dvojbodka
		call	wm_space
		call	wm_B
		call	wm_O
		call	wm_D
		call	wm_L
		call	wm_E
		call	wm_V
		call	wm_E
		call	wm_L
		call	wm_2
		call	wm_1
		call	wm_0
		call	wm_dvojbodka
		movlw	'0'
		btfsc	v_fuse_h,D'003'
		movlw	'1'
		call	send_data
		movlw	'0'
		btfsc	v_fuse_h,D'002'
		movlw	'1'
		call	send_data
		movlw	'0'
		btfsc	v_fuse_h,D'001'
		movlw	'1'
		call	send_data

		call	w_fuse_mod_flag

		movlw	B'11000111'		;row: 2 / col: 7
		call	send_comm
		call	wm_R
		call	wm_S
		call	wm_T
		call	wm_D
		call	wm_I
		call	wm_S
		call	wm_B
		call	wm_L
		call	wm_space
		call	wm_space
		call	wm_space
		call	wm_dvojbodka
		movlw	'0'
		btfsc	v_fuse_h,D'000'
		movlw	'1'
		call	send_data
		return
;------------------------------------------------------------------------------
;------------------------------------------------------------------------------
w_att013_4	movlw	B'10000000'		;row: 1 / col: 0
		call	send_comm

		call	wm_L
		call	wm_o
		call	wm_w
		call	wm_dvojbodka
		call	wm_space
		call	wm_space
		call	wm_space
		call	wm_C
		call	wm_K
		call	wm_D
		call	wm_I
		call	wm_V
		call	wm_8
		call	wm_dvojbodka
		movlw	'0'
		btfsc	v_fuse_l,D'007'
		movlw	'1'
		call	send_data

		call	wm_space
		call	wm_C
		call	wm_K
		call	wm_O
		call	wm_U
		call	wm_T
		call	wm_dvojbodka
		call	wm_space
		movlw	'0'
		btfsc	v_fuse_l,D'006'
		movlw	'1'
		call	send_data

		call	w_fuse_mod_flag

		movlw	B'11000010'		;row: 2 / col: 2
		call	send_comm
		call	wm_C
		call	wm_K
		call	wm_S
		call	wm_E
		call	wm_L
		call	wm_3
		call	wm_minus
		call	wm_0
		call	wm_dvojbodka
		movlw	'0'
		btfsc	v_fuse_l,D'003'
		movlw	'1'
		call	send_data
		movlw	'0'
		btfsc	v_fuse_l,D'002'
		movlw	'1'
		call	send_data
		movlw	'0'
		btfsc	v_fuse_l,D'001'
		movlw	'1'
		call	send_data
		movlw	'0'
		btfsc	v_fuse_l,D'000'
		movlw	'1'
		call	send_data

		call	wm_space
		call	wm_S
		call	wm_U
		call	wm_T
		call	wm_1
		call	wm_0
		call	wm_dvojbodka
		movlw	'0'
		btfsc	v_fuse_l,D'005'
		movlw	'1'
		call	send_data
		movlw	'0'
		btfsc	v_fuse_l,D'004'
		movlw	'1'
		call	send_data
		return
;------------------------------------------------------------------------------
;------------------------------------------------------------------------------
w_wf_info	movlw	B'10000000'		;row: 1 / col: 0
		call	send_comm

		call	wm_W
		call	wm_r
		call	wm_i
		call	wm_t
		call	wm_i
		call	wm_n
		call	wm_g
		call	wm_space
		call	wm_f
		call	wm_u
		call	wm_s
		call	wm_e
		call	wm_bodka
		call	wm_bodka
		call	wm_bodka
		return

;------------------------------------------------------------------------------
;------------------------------------------------------------------------------
w_att113_ia	nop
;     command:           '00001DCB' (DISPLAY ON/OFF)
;                              D = 1	--> Display on
;			       D = 0	--> Display off
;           		 '00001DCB'
;                               C = 1	--> Cursor on
;				C = 0	--> Cursor off
;           		 '00001DCB'
;				 B = 1	--> Blinking on
;				 B = 0	--> Blinking off
		movlw	B'00001111'
		call	send_comm
		return
;------------------------------------------------------------------------------
w_att113_id	nop
;     command:           '00001DCB' (DISPLAY ON/OFF)
;                              D = 1	--> Display on
;			       D = 0	--> Display off
;           		 '00001DCB'
;                               C = 1	--> Cursor on
;				C = 0	--> Cursor off
;           		 '00001DCB'
;				 B = 1	--> Blinking on
;				 B = 0	--> Blinking off
		movlw	B'00001100'
		call	send_comm
		return

;------------------------------------------------------------------------------
w_att113	movlw	B'10000000'		;row: 1 / col: 0
		call	send_comm

		call	wm_F
		call	wm_minus
		call	wm_L
		call	wm_o
		call	wm_w
		call	wm_dvojbodka
		call	wm_space
		movf	v_fuse_l,0
		movwf	kbd_buffer
		call	write_hex
		call	wm_H
;
		call	wm_space
		call	wm_space
		call	wm_space
		call	wm_F
		call	wm_minus
		call	wm_H
		call	wm_i
		call	wm_g
		call	wm_h
		call	wm_dvojbodka
		call	wm_space
		movf	v_fuse_h,0
		movwf	kbd_buffer
		call	write_hex
		call	wm_H
;
		movf	v_att_chip,0
		xorlw	H'08'
		btfsc	STATUS,Z
		goto	w_att13c4		;no extended fuse for ATtiny26

		movlw	B'11000000'		;row: 2 / col: 0
		call	send_comm
		call	wm_F
		call	wm_minus
		call	wm_E
		call	wm_x
		call	wm_t
		call	wm_dvojbodka
		call	wm_space
		movf	v_fuse_e,0
		movwf	kbd_buffer
		call	write_hex
		call	wm_H
;
w_att13c4	movlw	B'11001111'		;row: 2 / col: 15
		call	send_comm
;
		call	wm_L
		call	wm_o
		call	wm_c
		call	wm_k
		call	wm_dvojbodka
		call	wm_space
		movf	v_lock_b,0
		movwf	kbd_buffer
		call	write_hex
		call	wm_H

		return
;------------------------------------------------------------------------------
w_att011	movlw	B'10000000'		;row: 1 / col: 0
		call	send_comm

		call	wm_E
		call	wm_r
		call	wm_a
		call	wm_s
		call	wm_i
		call	wm_n
		call	wm_g
		call	wm_bodka
		call	wm_bodka
		call	wm_bodka
		return
;------------------------------------------------------------------------------
;------------------------------------------------------------------------------
;------------------------------------------------------------------------------
;------------------------------------------------------------------------------
;in W is the command to be send to display (posintion in display)
;output will be shifted to the variable		v_b_in
;value of the variable				v_comm will
;							ESC		01
;							ENTER		02
;							F10		03
;							other		FF
;------------------------------------------------------------------------------
hex_input	movwf	v_cursor
		movwf	tmp_ot

		movf	v_b_in,0
		movwf	b_ax_h
		movlw	B'11110000'
		andwf	b_ax_h,1
		rrf	b_ax_h,1
		rrf	b_ax_h,1
		rrf	b_ax_h,1
		rrf	b_ax_h,1

		PAGESEL	PAGE0
		call	sa_key
		PAGESEL	PAGE1
		movf	v_comm,0
		btfss	STATUS,Z
;exit - something happens
		goto	hi_exit_h

		movf	b_ax,0
		call	send_data
		call	hi_trans_h

		incf	v_cursor,0
		movwf	tmp_ot

		movf	v_b_in,0
		movwf	b_ax_h
		movlw	B'00001111'
		andwf	b_ax_h,1

		PAGESEL	PAGE0
		call	sa_key
		PAGESEL	PAGE1
		movf	v_comm,0
		btfss	STATUS,Z
		goto	hi_exit_l

		movf	b_ax,0
		call	send_data
		call	hi_trans_l
		return

;now we have to translate the value back to HEX

hi_exit_h	call	hi_trans_h
		return

hi_exit_l	call	hi_trans_l
		return


hi_trans_h	rlf	b_ax_h,1
		rlf	b_ax_h,1
		rlf	b_ax_h,1
		rlf	b_ax_h,1
		movlw	B'11110000'
		andwf	b_ax_h,1
		movlw	B'00001111'
		andwf	v_b_in,1
		movf	b_ax_h,0
		addwf	v_b_in,1
		return

hi_trans_l	movlw	B'00001111'
		andwf	b_ax_h,1
		movlw	B'11110000'
		andwf	v_b_in,1
		movf	b_ax_h,0
		addwf	v_b_in,1
		return

;------------------------------------------------------------------------------
;------------------------------------------------------------------------------
w_att_119_i	movlw	B'10000000'
		call	send_comm

		call	wm_C
		call	wm_o
		call	wm_m
		call	wm_p
		call	wm_a
		call	wm_r
		call	wm_i
		call	wm_n
		call	wm_g
		call	wm_space
		call	wm_P
		call	wm_g
		call	wm_dvojbodka

		goto	w_at01m_g1
;-----------------------------------------------------------------------------
w_att_119_ok	movlw	B'10000000'
		call	send_comm

		call	wm_C
		call	wm_o
		call	wm_m
		call	wm_p
		call	wm_a
		call	wm_r
		call	wm_e
		call	wm_space
		call	wm_o
		call	wm_k
		return
;-----------------------------------------------------------------------------
w_att_119_err	movlw	B'10000000'
		call	send_comm

		call	wm_C
		call	wm_o
		call	wm_m
		call	wm_p
		call	wm_a
		call	wm_r
		call	wm_e
		call	wm_space
		call	wm_e
		call	wm_r
		call	wm_r
		call	wm_o
		call	wm_r
		call	wm_space
		call	wm_a
		call	wm_t
		call	wm_dvojbodka

		movf	v_ad1,0
		movwf	kbd_buffer
		call	write_hex

		movf	v_ad0,0
		movwf	kbd_buffer
		call	write_hex

;------------------------------------
		movlw	B'11000000'
		call	send_comm
;
		call	wm_E
		call	wm_E
		call	wm_P
		call	wm_R
		call	wm_O
		call	wm_M
		call	wm_dvojbodka
		movf	v_d1,0
		movwf	kbd_buffer
		call	write_hex
		call	wm_minus
		movf	v_d0,0
		movwf	kbd_buffer
		call	write_hex

		call	wm_hvizdicka
;
		call	wm_F
		call	wm_l
		call	wm_a
		call	wm_s
		call	wm_h
		call	wm_dvojbodka
		movf	v_att_fh1,0
		movwf	kbd_buffer
		call	write_hex
		call	wm_minus
		movf	v_att_fl1,0
		movwf	kbd_buffer
		call	write_hex

		return
;------------------------------------------------------------------------------
;------------------------------------------------------------------------------
;------------------------------------------------------------------------------
;------------------------------------------------------------------------------
;------------------------------------------------------------------------------
;------------------------------------------------------------------------------
;------------------------------------------------------------------------------




;XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
;XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
; ATtiny2313A communication routines --- PAGE 2 --------------------------------
;XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
;XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX


ORG 1000
PAGE2
;xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx
;x                       initialization                                        x
;xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx
att_start	nop

		call	init_ports
;---------------------------------------------------------------------

		call	init_if

;in the beginning identify inserted chip -----
		call	att_chip01
;---------------------------------------------

att_001		nop
		PAGESEL	PAGE1
		call	disp_clr1
		call	disp_clr2
		call	w_att001
		PAGESEL	PAGE2

		call	leave_prog		;exit the programming mode completely


att_002		clrf	kbd_buffer
		PAGESEL	PAGE0
		call	get_datafk
		PAGESEL	PAGE2

		movf	kbd_buffer,0
		xorlw	H'2D'		;R = read few bytes of the Program Memory
		btfsc	STATUS,Z
		goto	att_017

		movf	kbd_buffer,0
		xorlw	H'1D'		;W = write few bytes of the Program Memory
		btfsc	STATUS,Z
		goto	att_009r

		movf	kbd_buffer,0
		xorlw	H'24'		;E = Erase chip
		btfsc	STATUS,Z
		goto	att_011

		movf	kbd_buffer,0
		xorlw	H'2B'		;F = Fuse
		btfsc	STATUS,Z
		goto	att_113

		movf	kbd_buffer,0
		xorlw	H'2A'		;V = Verify
		btfsc	STATUS,Z
		goto	att_119

		movf	kbd_buffer,0
		xorlw	H'76'		;ESC - back to start menu
		btfsc	STATUS,Z
		goto	ext2m001

		goto	att_002

;-------------------------------------------------------------------------------
ext2m001	nop
		PAGESEL	PAGE0
		goto	m001


;-------------------------------------------------------------------------------

;-------------------------------------------------------------------------------
;-------------------------------------------------------------------------------
;    ERASE CHIP
;-------------------------------------------------------------------------------
;-------------------------------------------------------------------------------
att_011		nop
;
		PAGESEL	PAGE1
		call	disp_clr1
		call	disp_clr2
		call	w_att011
		PAGESEL	PAGE2

		call	att_init_prg

		movlw	B'10000000'		;ATtiny command: ERASE
		movwf	v_byte
		call	att_sgn_open

		bcf	PORTD,D'006'		;-WR := 0
		PAGESEL	PAGE0
		call	d55
		call	d55
		call	d55
		PAGESEL	PAGE2
		bsf	PORTD,D'006'

		call	leave_prog		;exit the programming mode completely

		goto	att_017

;-------------------------------------------------------------------------------
;-------------------------------------------------------------------------------
;    VERIFY PROGRAM MEMORY - compare content of EEPROM with Atmel Flash
;-------------------------------------------------------------------------------
;-------------------------------------------------------------------------------
att_119		nop
;
		PAGESEL	PAGE1
		call	disp_clr1
		call	disp_clr2
		call	w_att_119_i
		PAGESEL	PAGE2

;-----------------------------------------------
;EEPROM address pointer init
		clrf	v_ad0
		clrf	v_ad1

;init Atmel Flash address pointer
		movlw	D'000'
		movwf	v_mll
		movwf	v_mlh
;-----------------------------------------------
;open Atmel for reading
		call	a017_open

;open I2C EEPROM for read
		PAGESEL	PAGE0
		call	eep_i2c_open_r
		PAGESEL	PAGE2
;-----------------------------------------------

;load the Flash size into local variables ------

		movf	v_pages,0		;number of pages to be written
		movwf	v_att_pgs
		clrf	v_att_pgs_w

att_loop1v	movf	v_page_size,0		;page size
		movwf	v_att_cnt		;move "pages size" words at once

		PAGESEL	PAGE1
		call	w_at01_prog		;progress indication
		PAGESEL	PAGE2

att_loop2v	nop
;read 1 word (2xbyte) from EEPROM -------------
;result will be stored in v_d0--v_d1
;
		PAGESEL	PAGE0
		call	eep_i2c_read
		movf	kbd_buffer,0
		movwf	v_d0
		call	eep_i2c_read
		movf	kbd_buffer,0
		movwf	v_d1
		PAGESEL	PAGE2
;----------------------------------------------
;read 1 word from Atmel Flash
;result will be stored in v_att_fh1--v_att_fl1
;
		call	a017_read
;compare result ------------------------------
		movf	v_d1,0
		xorwf	v_att_fh1,0
		btfss	STATUS,Z
		goto	att_119_v_err

		movf	v_d0,0
		xorwf	v_att_fl1,0
		btfss	STATUS,Z
		goto	att_119_v_err
;compare ok
;next please....
;increment Flash and EEPROM address pointers ------
		call	a017_inc_a
		movlw	D'001'
		PAGESEL	PAGE0
		call	eeprom_inc_adn
		PAGESEL	PAGE2
;
		decfsz	v_att_cnt,1
		goto	att_loop2v

		incf	v_att_pgs_w,1
		decfsz	v_att_pgs,1
		goto	att_loop1v

		nop
;--------------------------------------------------
;if we reached this point, everthing went fine....
;
		PAGESEL	PAGE1
		call	disp_clr1
		call	disp_clr2
		call	w_att_119_ok
		PAGESEL	PAGE2


;-------------------------------------------------------------------------------------
;close I2C EEPROM communication -------------
att_119_close	nop
		PAGESEL	PAGE0
		call	eep_i2c_close
		PAGESEL	PAGE2
;leave the programming mode
		call	leave_prog		;exit the programming mode completely
;-------------------------------------------------------------------------------------

att_119_k1	clrf	kbd_buffer
		PAGESEL	PAGE0
		call	get_datafk
		PAGESEL	PAGE2

		movf	kbd_buffer,0
		xorlw	H'5A'		;ENTER = back to Atmel menu
		btfsc	STATUS,Z
		goto	att_001

		movf	kbd_buffer,0
		xorlw	H'76'		;ESC = back to Atmel menu
		btfsc	STATUS,Z
		goto	att_001

		goto	att_119_k1


;compare error - data are not the same ------------------------------
att_119_v_err	nop
		PAGESEL	PAGE1
		call	disp_clr1
		call	disp_clr2
		call	w_att_119_err
		PAGESEL	PAGE2

		goto	att_119_close
;-------------------------------------------------------------------------------
;-------------------------------------------------------------------------------
;    WRITE PROGRAM MEMORY - data from EEPROM
;-------------------------------------------------------------------------------
;-------------------------------------------------------------------------------
att_009r	nop
;
		PAGESEL	PAGE1
		call	disp_clr1
		call	disp_clr2
		call	w_at01m_prog
		PAGESEL	PAGE2

; ---------- enter programming mode ---------------------------------------

		call	att_init_prg

; ---------- ERASE the chip prior to write --------------------------------

		movlw	B'10000000'		;ATtiny command: ERASE
		movwf	v_byte
		call	att_sgn_open

		bcf	PORTD,D'006'		;-WR := 0
		PAGESEL	PAGE0
		call	d2			;wait a little bit longer - just for sure (we are currently ignoring RDY/-BSY)
		PAGESEL	PAGE2
		bsf	PORTD,D'006'
; --------------------------------------------------------------------------

		movlw	B'00010000'		;ATtiny command: write program memory
		movwf	v_byte
		call	att_sgn_open

;---------------------------------------
;open I2C EEPROM for read
		clrf	v_ad0
		clrf	v_ad1

		PAGESEL	PAGE0
		call	eep_i2c_open_r
		PAGESEL	PAGE2
;---------------------------------------

;---------------------------------------------------------------------------
;------- v_att_add_h ... v_ad1 ---------------------------------------------
;---------------------------------------------------------------------------
		clrf	v_ad1
		clrf	v_att_add_h

		movlw	H'00'
		movwf	v_att_pga		;address within the page ( = 00H )


		movf	v_pages,0		;number of pages to be written
		movwf	v_att_pgs
		clrf	v_att_pgs_w


att_loop1	movf	v_page_size,0		;page size
		movwf	v_att_cnt		;move "pages size" words at once

		PAGESEL	PAGE1
		call	w_at01_prog		;display progress
		PAGESEL	PAGE2

; start I2C communication with EEPROM

att_loop2	nop

;read byte from EEPROM

		PAGESEL	PAGE0
		call	eep_i2c_read
		movf	kbd_buffer,0
		movwf	v_d0
		call	eep_i2c_read
		movf	kbd_buffer,0
		movwf	v_d1
		PAGESEL	PAGE2

; write data to ATMEL
		movf	v_att_pga,0		;address
		movwf	v_att_add
		call	att_add_L

		movf	v_d0,0
		movwf	v_att_add
		call	att_dat_L

		movf	v_d1,0
		movwf	v_att_add
		call	att_dat_H

;----------------------------------------------------------------------------------------------------
;latch data (ATtiny88 only) - ignored by other Tinys (actually in HW not connected to them at all)
		bsf	PORTD,D'007'		;BS1   = 1
		bsf	PORTC,D'000'		;PAGEL = 1
		call	d55p2
		bcf	PORTC,D'000'		;PAGEL = 0
		bcf	PORTD,D'007'		;BS1   = 0
;----------------------------------------------------------------------------------------------------

		incf	v_att_pga,1

		decfsz	v_att_cnt,1
		goto	att_loop2

; ----------- the complete page has been written....


;load address high
		movf	v_att_add_h,0
		movwf	v_att_add
		call	att_add_H

;program write
		bcf	PORTD,D'006'		;-WR := 0
		PAGESEL	PAGE0
		call	d2			;wait a little bit longer - just for sure (we are currently ignoring RDY/-BSY)
		PAGESEL	PAGE2
		bsf	PORTD,D'006'

		call	d55p2

;check if there is a need to increment address high ----
		movf	v_att_pga,0
		btfsc	STATUS,Z
		incf	v_att_add_h,1		;increment address high; when address low overflows
;-------------------------------------------------------

		incf	v_att_pgs_w,1
		decfsz	v_att_pgs,1
		goto	att_loop1

;End Page Programming
		movlw	B'00000000'		;ATtiny command: NOP
		movwf	v_byte
		call	att_sgn_open

		call	d55p2


;close I2C EEPROM communication -------------
		PAGESEL	PAGE0
		call	eep_i2c_close
		PAGESEL	PAGE2
;--------------------------------------------
		goto	att_017



;XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
;-------------------------------------------------------------------------------
;-------------------------------------------------------------------------------
;    READ PROGRAM MEMORY routines
;-------------------------------------------------------------------------------
;-------------------------------------------------------------------------------
;XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
;
;-----------------------------------------
;OPEN for READING
;-----------------------------------------
a017_open	nop
;----------------------------------------------
		call	att_init_prg
		movlw	B'00000010'		;ATtiny command: read program memory
		movwf	v_byte
		call	att_sgn_open

		return

;-----------------------------------------
;READ a WORD from the address v_mlh-v_mll
;-----------------------------------------
;----------------------------------------------
;load the flash address to the controller
a017_read	movf	v_mlh,0
		movwf	v_att_add
		call	att_add_H
		movf	v_mll,0
		movwf	v_att_add
		call	att_add_L
;----------------------------------------------
;read the content of the memory location
		call	att_pm_read
		movf	v_att_fh,0
		movwf	v_att_fh1
		movf	v_att_fl,0
		movwf	v_att_fl1

		return
;----------------------------------------------
;increment the flash pointer
a017_inc_a	incf	v_mll,1
		btfss	STATUS,Z
		return
		incf	v_mlh,1
		btfss	STATUS,Z
		return
		clrf	v_mlh
		return
;----------------------------------------------
;decrement the flash pointer
a017_dec_a6	movlw	D'012'
		movwf	TIMER1

a017_dec_a6_l	decf	v_mll,1
		movlw	H'FF'
		xorwf	v_mll,0
		btfsc	STATUS,Z
		decf	v_mlh,1
		decfsz	TIMER1,1
		goto	a017_dec_a6_l
		return
;----------------------------------------------
;increment the flash pointer - add 256
a017_inc_pg	incf	v_mlh,1
		return
;----------------------------------------------
;decrement the flash pointer - dec 256
a017_dec_pg	decf	v_mlh,1
		return
;-------------------------------------------------------------------------------
;-------------------------------------------------------------------------------
;-------------------------------------------------------------------------------
att_017		nop
;
;start reading from flash location: 0000 ------
		movlw	D'000'
		movwf	v_mll
		movwf	v_mlh

		call	a017_open

att017_l3	movlw	D'002'
		movwf	v_tmp6

		PAGESEL	PAGE1
		call	disp_clr1
		call	disp_clr2
		movlw	B'10000000'
		call	send_comm
		PAGESEL	PAGE2

att017_loop	nop
		PAGESEL	PAGE1
		movf	v_mlh,0
		movwf	kbd_buffer
		call	write_hex
		movf	v_mll,0
		movwf	kbd_buffer
		call	write_hex
		call	wm_dvojbodka
		PAGESEL	PAGE2

		movlw	D'003'
		movwf	v_tmp5

att017_l2	call	a017_read

		PAGESEL	PAGE1
		call	wm_space
		movf	v_att_fh1,0
		movwf	kbd_buffer
		call	write_hex
		call	wm_minus
		movf	v_att_fl1,0
		movwf	kbd_buffer
		call	write_hex
		PAGESEL	PAGE2

		call	a017_inc_a

		decfsz	v_tmp5,1
		goto	att017_l2

		nop
		PAGESEL	PAGE1
		movlw	B'11000000'
		call	send_comm
		PAGESEL	PAGE2

		decfsz	v_tmp6,1
		goto	att017_loop

;-------------------------------------
att_flm_l1	clrf	kbd_buffer
		PAGESEL	PAGE0
		call	get_datafk
		PAGESEL	PAGE2

		movf	kbd_buffer,0
		xorlw	H'5A'		;ENTER = forward
		btfsc	STATUS,Z
		goto	att017_l3

		movf	kbd_buffer,0
		xorlw	H'74'		;-> = forward
		btfsc	STATUS,Z
		goto	att017_l3

		movf	kbd_buffer,0
		xorlw	H'75'		;^ (arrow up) = page forward
		btfsc	STATUS,Z
		goto	att017_fwd2

		movf	kbd_buffer,0
		xorlw	H'6B'		;-> = backward
		btfsc	STATUS,Z
		goto	att017_back

		movf	kbd_buffer,0
		xorlw	H'72'		;v (arrow down) = page back
		btfsc	STATUS,Z
		goto	att017_bck2

		movf	kbd_buffer,0
		xorlw	H'76'		;ESC = back to Atmel menu
		btfsc	STATUS,Z
		goto	att_001

		goto	att_flm_l1


;---------------------------------------------------------
att017_back	call	a017_dec_a6
		goto	att017_l3

att017_fwd2	call	a017_inc_pg
		goto	att017_l3

att017_bck2	call	a017_dec_pg
		goto	att017_l3
;-------------------------------------------------------------------------------
;-------------------------------------------------------------------------------
;---------------------------------------------------------
;---------------------------------------------------------
;display the signature
		nop
		PAGESEL	PAGE1
		call	w_att_sgn
		PAGESEL	PAGE2

att_004		clrf	kbd_buffer
		PAGESEL	PAGE0
		call	get_datafk
		PAGESEL	PAGE2

		movf	kbd_buffer,0
		xorlw	H'76'		;ESC - back to start menu
		btfsc	STATUS,Z
		goto	att_001

		goto	att_004
;-------------------------------------------------------------------------------
;-------------------------------------------------------------------------------
init_if		bcf	PORTB,D'003'
		bcf	PORTB,D'004'
		bcf	PORTB,D'005'
		bcf	PORTB,D'006'
		bcf	PORTB,D'007'

		bsf	PORTD,D'005'	;-OE
		bsf	PORTD,D'006'	;-WR
		bcf	PORTD,D'007'

		bcf	PORTC,D'000'	;PAGEL	28pin
		bcf	PORTC,D'001'	;BS2	28pin

		return
;-------------------------------------------------------------------------------
;-------------------------------------------------------------------------------
;---------------------------------------------------------------------------------------
;---------------------------------------------------------------------------------------
att_init_prg	nop
		call	init_ports

		call	init_if

		PAGESEL	PAGE0
		call	d2
		PAGESEL	PAGE2

		bcf	PORTD,D'006'		;-WR
		PAGESEL	PAGE0
		call	d2
		PAGESEL	PAGE2
;
;switch the power on
		bsf	PORTB,D'007'		;power ON

;now wait a little bit...
		call	d55p2		;but not too sooo long! (it can be a problem)

;connect RESET to +12V
		bsf	PORTB,D'005'		;-RESET
;wait a little bit again...
		PAGESEL	PAGE0
		call	d2
		PAGESEL	PAGE2
;now we have entered the programming mode ----

		bsf	PORTD,D'006'		;deactivate -WR

;wait a little bit again...
		call	d55p2

		return
;---------------------------------------------------------------------------------------
;---------------------------------------------------------------------------------------
;load the command
;input: command in register v_byte
;---------------------------------------------------------------------------------------
;---------------------------------------------------------------------------------------
att_sgn_open	movlw	B'01000000'		;address of the local PCF8574 device
		movwf	v_pcf_add
		PAGESEL	PAGE0		
		call	pcf_i2c_write		;AT command has been stored in "v_byte" prio to call
		PAGESEL	PAGE2
;
;XA0 := 0; XA1 = 1 --> load command
		bcf	PORTB,D'003'		;XA0 = 0
		bsf	PORTB,D'004'		;XA1 = 1
;BS1 := 0
		bcf	PORTD,D'007'		;BS1  = 0
;now we generate a positive pulse on XTAL1
		bsf	PORTB,D'006'

		call	d55p2

		bcf	PORTB,D'006'

		call	d55p2

		return
;---------------------------------------------------------------------------------------
;---------------------------------------------------------------------------------------
;load data HIGH
;input: data in register v_att_add
;---------------------------------------------------------------------------------------
;---------------------------------------------------------------------------------------
att_dat_H	movlw	B'01000000'		;address of the local PCF8574 device
		movwf	v_pcf_add
		movf	v_att_add,0		;data
		movwf	v_byte
		PAGESEL	PAGE0		
		call	pcf_i2c_write
		PAGESEL	PAGE2
;
;XA0 := 1; XA1 = 0 --> load data
		bsf	PORTB,D'003'		;XA0 = 1
		bcf	PORTB,D'004'		;XA1 = 0
;BS1 := 1 (high data)
		bsf	PORTD,D'007'		;BS1  = 1

;now we generate a positive pulse on XTAL1
		bsf	PORTB,D'006'

		call	d55p2

		bcf	PORTB,D'006'

		call	d55p2

		bcf	PORTD,D'007'		;BS1  = 0

		call	d55p2

		return
;---------------------------------------------------------------------------------------
;---------------------------------------------------------------------------------------
;load data LOW
;input: data in register v_att_add
;---------------------------------------------------------------------------------------
;---------------------------------------------------------------------------------------
att_dat_L	movlw	B'01000000'		;address of the local PCF8574 device
		movwf	v_pcf_add
		movf	v_att_add,0		;data
		movwf	v_byte
		PAGESEL	PAGE0		
		call	pcf_i2c_write
		PAGESEL	PAGE2
;
;XA0 := 1; XA1 = 0 --> load data
		bsf	PORTB,D'003'		;XA0 = 1
		bcf	PORTB,D'004'		;XA1 = 0
;BS1 := 0 (low data)
		bcf	PORTD,D'007'		;BS1  = 0

;now we generate a positive pulse on XTAL1
		bsf	PORTB,D'006'

		call	d55p2

		bcf	PORTB,D'006'

		call	d55p2

		return
;---------------------------------------------------------------------------------------
;---------------------------------------------------------------------------------------
;load address LOW
;input: address in register v_att_add
;---------------------------------------------------------------------------------------
;---------------------------------------------------------------------------------------
att_add_L	movlw	B'01000000'		;address of the local PCF8574 device
		movwf	v_pcf_add
		movf	v_att_add,0		;ATtiny address
		movwf	v_byte
		PAGESEL	PAGE0		
		call	pcf_i2c_write
		PAGESEL	PAGE2
;
;XA0 := 0; XA1 = 0 --> load address
		bcf	PORTB,D'003'		;XA0 = 0
		bcf	PORTB,D'004'		;XA1 = 0
;BS1 := 0 (low address)
		bcf	PORTD,D'007'		;BS1  = 0

;now we generate a positive pulse on XTAL1
		bsf	PORTB,D'006'

		call	d55p2

		bcf	PORTB,D'006'


		call	d55p2


		return
;---------------------------------------------------------------------------------------
;---------------------------------------------------------------------------------------
;load address HIGH
;
;input: address in register v_att_add
;---------------------------------------------------------------------------------------
;---------------------------------------------------------------------------------------
att_add_H	movlw	B'01000000'		;address of the local PCF8574 device
		movwf	v_pcf_add
		movf	v_att_add,0		;ATtiny address
		movwf	v_byte
		PAGESEL	PAGE0		
		call	pcf_i2c_write
		PAGESEL	PAGE2
;
;XA0 := 0; XA1 = 0 --> load address
		bcf	PORTB,D'003'		;XA0 = 0
		bcf	PORTB,D'004'		;XA1 = 0
;BS1 := 1 (high address)
		bsf	PORTD,D'007'		;BS1  = 1

;now we generate a positive pulse on XTAL1
		bsf	PORTB,D'006'
		call	d55p2

		bcf	PORTB,D'006'

		call	d55p2

		return
;---------------------------------------------------------------------------------------
;---------------------------------------------------------------------------------------
;read signature
;---------------------------------------------------------------------------------------
;---------------------------------------------------------------------------------------
att_sgn_read	nop

;read the signature...
;2: load address
		call	att_add_L
;
		bcf	PORTD,D'007'		;set BS1 to 0
;
;now we can read the data, BUT:
;before reading the data we have to put all outputs of the PCF8574 device to "1"
		movlw	B'01000000'		;address of the local PCF8574 device
		movwf	v_pcf_add
		movlw	B'11111111'		;init to 1
		movwf	v_byte
		PAGESEL	PAGE0
		call	pcf_i2c_write
		bcf	PORTD,D'005'		;-OE := 0
		call	d55
		call	pcf_i2c_read
		PAGESEL	PAGE2

		bsf	PORTD,D'005'		;disable -OE again

		bsf	PORTD,D'007'		;set BS1 to 1
		PAGESEL	PAGE0
		call	d55
		PAGESEL	PAGE2
		bcf	PORTD,D'007'		;set BS1 to 0

		return

;---------------------------------------------------------------------------------------
;---------------------------------------------------------------------------------------
;read addressed FLASH location
;
;output is stored in registers: v_att_fl & v_att_fh
;---------------------------------------------------------------------------------------
;---------------------------------------------------------------------------------------
att_pm_read	nop
;
;now we can read the data, BUT:
;before reading the data we have to put all outputs of the PCF8574 device to "1"
		movlw	B'01000000'		;address of the local PCF8574 device
		movwf	v_pcf_add
		movlw	B'11111111'		;init to 1
		movwf	v_byte
		PAGESEL	PAGE0
		call	pcf_i2c_write
		bcf	PORTD,D'007'		;set BS1 to 0
		bcf	PORTD,D'005'		;-OE := 0
		call	d55
		call	pcf_i2c_read
		PAGESEL	PAGE2

		movf	v_byte,0
		movwf	v_att_fl

		bsf	PORTD,D'007'		;set BS1 to 1
		PAGESEL	PAGE0
		call	d55
		movlw	B'01000000'		;address of the local PCF8574 device
		movwf	v_pcf_add
		movlw	B'11111111'		;init to 1
		movwf	v_byte
		call	pcf_i2c_write
		call	d55
		call	pcf_i2c_read
		PAGESEL	PAGE2

		movf	v_byte,0
		movwf	v_att_fh

		bsf	PORTD,D'005'		;disable -OE again
		bcf	PORTD,D'007'		;set BS1 to 0

		return

;-------------------------------------------------------------------------------
;XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
;-------------------------------------------------------------------------------
att_chip01	movlw	H'FE'
		movwf	v_att_chip		;default = not known

;try to get the signature from ATtiny2313 / 2313A / 4313
		call	att_003
;---------------------------------------------------------------------
		movf	v_att_s0,0
		xorlw	H'1E'
		btfss	STATUS,Z
		goto	att_c_n2
		movf	v_att_s1,0
		xorlw	H'91'
		btfss	STATUS,Z
		goto	att_c_n2
		movf	v_att_s2,0
		xorlw	H'0A'
		btfss	STATUS,Z
		goto	att_c_n2
;
		movlw	H'01'			;ATtiny2313 or ATtiny2313A
		movwf	v_att_chip
		movlw	D'016'			;Page size
		movwf	v_page_size
		movlw	D'064'			;number of pages
		movwf	v_pages
		goto	att_c_n99
;---------------------------------------------------------------------
att_c_n2	movf	v_att_s0,0
		xorlw	H'1E'
		btfss	STATUS,Z
		goto	att_c_n3
		movf	v_att_s1,0
		xorlw	H'92'
		btfss	STATUS,Z
		goto	att_c_n3
		movf	v_att_s2,0
		xorlw	H'0D'
		btfss	STATUS,Z
		goto	att_c_n3
;
		movlw	H'02'			;ATtiny4313
		movwf	v_att_chip
		movlw	D'032'			;Page size
		movwf	v_page_size
		movlw	D'064'			;number of pages
		movwf	v_pages
		goto	att_c_n99
;---------------------------------------------------------------------
att_c_n3	movf	v_att_s0,0
		xorlw	H'1E'
		btfss	STATUS,Z
		goto	att_c_n3b
		movf	v_att_s1,0
		xorlw	H'92'
		btfss	STATUS,Z
		goto	att_c_n3b
		movf	v_att_s2,0
		xorlw	H'09'
		btfss	STATUS,Z
		goto	att_c_n3b
;
		movlw	H'05'			;ATtiny48
		movwf	v_att_chip
		movlw	D'032'			;Page size
		movwf	v_page_size
		movlw	D'064'			;number of pages
		movwf	v_pages
		goto	att_c_n99
;---------------------------------------------------------------------
att_c_n3b	movf	v_att_s0,0
		xorlw	H'1E'
		btfss	STATUS,Z
		goto	att_c_n4
		movf	v_att_s1,0
		xorlw	H'93'
		btfss	STATUS,Z
		goto	att_c_n4
		movf	v_att_s2,0
		xorlw	H'11'
		btfss	STATUS,Z
		goto	att_c_n4
;
		movlw	H'03'			;ATtiny88
		movwf	v_att_chip
		movlw	D'032'			;Page size
		movwf	v_page_size
		movlw	D'128'			;number of pages
		movwf	v_pages
		goto	att_c_n99
;---------------------------------------------------------------------
att_c_n4	movf	v_att_s0,0
		xorlw	H'1E'
		btfss	STATUS,Z
		goto	att_c_n5
		movf	v_att_s1,0
		xorlw	H'92'
		btfss	STATUS,Z
		goto	att_c_n5
		movf	v_att_s2,0
		xorlw	H'08'
		btfss	STATUS,Z
		goto	att_c_n5
;
		movlw	H'04'			;ATtiny461A
		movwf	v_att_chip
		movlw	D'032'			;Page size
		movwf	v_page_size
		movlw	D'064'			;number of pages
		movwf	v_pages
		goto	att_c_n99
;---------------------------------------------------------------------
att_c_n5	movf	v_att_s0,0
		xorlw	H'1E'
		btfss	STATUS,Z
		goto	att_c_n6
		movf	v_att_s1,0
		xorlw	H'91'
		btfss	STATUS,Z
		goto	att_c_n6
		movf	v_att_s2,0
		xorlw	H'0C'
		btfss	STATUS,Z
		goto	att_c_n6
;
		movlw	H'06'			;ATtiny261A
		movwf	v_att_chip
		movlw	D'016'			;Page size
		movwf	v_page_size
		movlw	D'064'			;number of pages
		movwf	v_pages
		goto	att_c_n99
;---------------------------------------------------------------------
att_c_n6	movf	v_att_s0,0
		xorlw	H'1E'
		btfss	STATUS,Z
		goto	att_c_n7
		movf	v_att_s1,0
		xorlw	H'93'
		btfss	STATUS,Z
		goto	att_c_n7
		movf	v_att_s2,0
		xorlw	H'0D'
		btfss	STATUS,Z
		goto	att_c_n7
;
		movlw	H'07'			;ATtiny861A
		movwf	v_att_chip
		movlw	D'032'			;Page size
		movwf	v_page_size
		movlw	D'128'			;number of pages
		movwf	v_pages
		goto	att_c_n99
;---------------------------------------------------------------------
att_c_n7	movf	v_att_s0,0
		xorlw	H'1E'
		btfss	STATUS,Z
		goto	att_c_n8
		movf	v_att_s1,0
		xorlw	H'91'
		btfss	STATUS,Z
		goto	att_c_n8
		movf	v_att_s2,0
		xorlw	H'09'
		btfss	STATUS,Z
		goto	att_c_n8
;
		movlw	H'08'			;ATtiny26
		movwf	v_att_chip
		movlw	D'016'			;Page size
		movwf	v_page_size
		movlw	D'064'			;number of pages
		movwf	v_pages
		goto	att_c_n99
;---------------------------------------------------------------------
att_c_n8	movf	v_att_s0,0
		xorlw	H'1E'
		btfss	STATUS,Z
		goto	att_c_n9
		movf	v_att_s1,0
		xorlw	H'92'
		btfss	STATUS,Z
		goto	att_c_n9
		movf	v_att_s2,0
		xorlw	H'05'
		btfss	STATUS,Z
		goto	att_c_n9
;
		movlw	H'11'			;ATmega48A
		movwf	v_att_chip
		movlw	D'032'			;Page size
		movwf	v_page_size
		movlw	D'064'			;number of pages
		movwf	v_pages
		goto	att_c_n99
;---------------------------------------------------------------------
att_c_n9	movf	v_att_s0,0
		xorlw	H'1E'
		btfss	STATUS,Z
		goto	att_c_n10
		movf	v_att_s1,0
		xorlw	H'92'
		btfss	STATUS,Z
		goto	att_c_n10
		movf	v_att_s2,0
		xorlw	H'0A'
		btfss	STATUS,Z
		goto	att_c_n10
;
		movlw	H'12'			;ATmega48PA
		movwf	v_att_chip
		movlw	D'032'			;Page size
		movwf	v_page_size
		movlw	D'064'			;number of pages
		movwf	v_pages
		goto	att_c_n99
;---------------------------------------------------------------------
att_c_n10	movf	v_att_s0,0
		xorlw	H'1E'
		btfss	STATUS,Z
		goto	att_c_n11
		movf	v_att_s1,0
		xorlw	H'93'
		btfss	STATUS,Z
		goto	att_c_n11
		movf	v_att_s2,0
		xorlw	H'0A'
		btfss	STATUS,Z
		goto	att_c_n11
;
		movlw	H'13'			;ATmega88A
		movwf	v_att_chip
		movlw	D'032'			;Page size
		movwf	v_page_size
		movlw	D'128'			;number of pages
		movwf	v_pages
		goto	att_c_n99
;---------------------------------------------------------------------
att_c_n11	movf	v_att_s0,0
		xorlw	H'1E'
		btfss	STATUS,Z
		goto	att_c_n12
		movf	v_att_s1,0
		xorlw	H'93'
		btfss	STATUS,Z
		goto	att_c_n12
		movf	v_att_s2,0
		xorlw	H'0F'
		btfss	STATUS,Z
		goto	att_c_n12
;
		movlw	H'14'			;ATmega88PA
		movwf	v_att_chip
		movlw	D'032'			;Page size
		movwf	v_page_size
		movlw	D'128'			;number of pages
		movwf	v_pages
		goto	att_c_n99
;---------------------------------------------------------------------
att_c_n12	movf	v_att_s0,0
		xorlw	H'1E'
		btfss	STATUS,Z
		goto	att_c_n13
		movf	v_att_s1,0
		xorlw	H'94'
		btfss	STATUS,Z
		goto	att_c_n13
		movf	v_att_s2,0
		xorlw	H'06'
		btfss	STATUS,Z
		goto	att_c_n13
;
		movlw	H'15'			;ATmega168A
		movwf	v_att_chip
		movlw	D'064'			;Page size
		movwf	v_page_size
		movlw	D'128'			;number of pages
		movwf	v_pages
		goto	att_c_n99
;---------------------------------------------------------------------
att_c_n13	movf	v_att_s0,0
		xorlw	H'1E'
		btfss	STATUS,Z
		goto	att_c_n14
		movf	v_att_s1,0
		xorlw	H'94'
		btfss	STATUS,Z
		goto	att_c_n14
		movf	v_att_s2,0
		xorlw	H'0B'
		btfss	STATUS,Z
		goto	att_c_n14
;
		movlw	H'16'			;ATmega168PA
		movwf	v_att_chip
		movlw	D'064'			;Page size
		movwf	v_page_size
		movlw	D'128'			;number of pages
		movwf	v_pages
		goto	att_c_n99
;---------------------------------------------------------------------
att_c_n14	movf	v_att_s0,0
		xorlw	H'1E'
		btfss	STATUS,Z
		goto	att_c_n15
		movf	v_att_s1,0
		xorlw	H'95'
		btfss	STATUS,Z
		goto	att_c_n15
		movf	v_att_s2,0
		xorlw	H'14'
		btfss	STATUS,Z
		goto	att_c_n15
;
		movlw	H'17'			;ATmega328
		movwf	v_att_chip
		movlw	D'064'			;Page size
		movwf	v_page_size
		movlw	D'000'			;number of pages = 256 --- dblchk!!!!
		movwf	v_pages
		goto	att_c_n99
;---------------------------------------------------------------------
att_c_n15	movf	v_att_s0,0
		xorlw	H'1E'
		btfss	STATUS,Z
		goto	att_c_n97
		movf	v_att_s1,0
		xorlw	H'95'
		btfss	STATUS,Z
		goto	att_c_n97
		movf	v_att_s2,0
		xorlw	H'0F'
		btfss	STATUS,Z
		goto	att_c_n97
;
		movlw	H'18'			;ATmega328P
		movwf	v_att_chip
		movlw	D'064'			;Page size
		movwf	v_page_size
		movlw	D'000'			;number of pages = 256 --- dblchk!!!!
		movwf	v_pages
		goto	att_c_n99

;---------------------------------------------------------------------
att_c_n97	nop
;

		movf	v_att_s0,0
		xorlw	H'FF'
		btfss	STATUS,Z
		goto	att_c_n99

		movlw	H'FF'
		movwf	v_att_chip		;no chip inserted

att_c_n99	nop

		call	leave_prog		;exit the programming mode completely

		return
;XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
;XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
; ATtiny2313 / ATtiny2313A / ATtiny4313
;XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
;XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
;-------------------------------------------------------------------------------
;-------------------------------------------------------------------------------
;         Read ATtiny2313A signature
;the signature for 2313A is expected to be: 1E 91 0A (see Page 191 of Datasheet)
;-------------------------------------------------------------------------------
;-------------------------------------------------------------------------------
att_003		nop
;---------------------------------------------------------
;---- start the programming mode -------------------------
;---------------------------------------------------------

		call	att_init_prg
		movlw	B'00001000'		;ATtiny command: read signature
		movwf	v_byte
		call	att_sgn_open

		movlw	D'000'
		movwf	v_att_add
		call	att_sgn_read
		movf	v_byte,0
		movwf	v_att_s0

		movlw	D'001'
		movwf	v_att_add
		call	att_sgn_read
		movf	v_byte,0
		movwf	v_att_s1

		movlw	D'002'
		movwf	v_att_add
		call	att_sgn_read
		movf	v_byte,0
		movwf	v_att_s2
;

;put RESET to 0V to exit the programming mode
		bcf	PORTB,D'005'

		return
;------------------------------------------------------------------------------
;------------------------------------------------------------------------------
;------------------------------------------------------------------------------
init_ports	bsf	STATUS,RP0	;Switch to register bank 1

;RB3 = XA0       (output)
;RB4 = XA1/BS2   (output)
;RB5 = -RESET    (output) -- putting RB5 to 1 will switch on 12V to the reset pin
;RB6 = XTAL1     (output)
;RB7 = Vcc       (output) -- putting RB7 to 1 will switch on the programming board

;RD4 = RDY/-BSY  (input)
;RD5 = -OE       (output)
;RD6 = -WR       (output)
;RD7 = BS1/PAGEL (output)

;RC0 = PAGEL (ATtiny88)
;RC1 = BS2   (ATtiny88)

		MOVLW	B'00000111'	;RB0 = keyboard clock; RB1 = kbd data
		MOVWF	TRISB

		MOVLW	B'00010000'	;PORTD = input/output: RD0-RD3 - LCD
		MOVWF	TRISD

		bcf	TRISC,D'000'
		bcf	TRISC,D'001'

		bcf	STATUS,RP0	;Switch to register bank 0

		return
;------------------------------------------------------------------------------
;------------------------------------------------------------------------------
;------------------------------------------------------------------------------
leave_prog	nop
;put RESET to 0V to exit the programming mode
		bcf	PORTB,D'005'
;switch the power off
		bcf	PORTB,D'007'		;power OFF

;put I/O to inputs to disconnect from the chip
		bsf	STATUS,RP0
		bsf	TRISB,D'006'
		bsf	TRISB,D'004'
		bsf	TRISB,D'003'

		bsf	TRISD,D'004'
		bsf	TRISD,D'005'
		bsf	TRISD,D'006'
		bsf	TRISD,D'007'

		bsf	TRISC,D'000'
		bsf	TRISC,D'001'

		movlw	D'255'
		movwf	TRISA
		bcf	STATUS,RP0

		return
;-------------------------------------------------------------------------------
;-------------------------------------------------------------------------------
; Get the Fuse data from 2313 device
;
;for now we are just using 2313 Fuse (release 0.11)
;for later releases we have to read the data based on the variable v_att_chip
;
;-------------------------------------------------------------------------------
;-------------------------------------------------------------------------------
read_fuse2313	call	att_init_prg
		movlw	B'00000100'		;ATtiny command: read Fuse and lock bits
		movwf	v_byte
		call	att_sgn_open
;Fuse Low
		bcf	PORTD,D'005'		;-OE
		bcf	PORTD,D'007'		;BS1
		bcf	PORTB,D'004'		;BS2 (20-pin)
		bcf	PORTC,D'001'		;BS2 (28-pin)
		PAGESEL	PAGE0
		call	pcf_i2c_read
		PAGESEL	PAGE2
		movf	v_byte,0
		movwf	v_fuse_l
;Fuse High
		bcf	PORTD,D'005'		;-OE
		bsf	PORTD,D'007'		;BS1
		bsf	PORTB,D'004'		;BS2 (20-pin)
		bsf	PORTC,D'001'		;BS2 (28-pin)
		PAGESEL	PAGE0
		call	pcf_i2c_read
		PAGESEL	PAGE2
		movf	v_byte,0
		movwf	v_fuse_h
;Fuse Extended
		bcf	PORTD,D'005'		;-OE
		bcf	PORTD,D'007'		;BS1
		bsf	PORTB,D'004'		;BS2 (20-pin)
		bsf	PORTC,D'001'		;BS2 (28-pin)
		PAGESEL	PAGE0
		call	pcf_i2c_read
		PAGESEL	PAGE2
		movf	v_byte,0
		movwf	v_fuse_e
;Lock Bits
		bcf	PORTD,D'005'		;-OE
		bsf	PORTD,D'007'		;BS1
		bcf	PORTB,D'004'		;BS2 (20-pin)
		bcf	PORTC,D'001'		;BS2 (28-pin)
		PAGESEL	PAGE0
		call	pcf_i2c_read
		PAGESEL	PAGE2
		movf	v_byte,0
		movwf	v_lock_b
;
		bsf	PORTD,D'005'		;-OE

		clrf	v_fuse_mod

		return



;XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
;-------------------------------------------------------------------------------
;-------------------------------------------------------------------------------
;    Fuse / Lock Maintenance
;    general
;-------------------------------------------------------------------------------
;-------------------------------------------------------------------------------
;XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
;FUSE EXTENDED screen
;incl. reading data from the device
;
att_113		call	read_fuse2313
;we have to read the Fuse data from the device first

		PAGESEL	PAGE1
		call	disp_clr1
		call	disp_clr2
		call	w_att113_ia
		PAGESEL	PAGE2

att_113_c7	nop
		PAGESEL	PAGE1
		call	w_att113
		PAGESEL	PAGE2

;input fuse low
		movf	v_fuse_l,0
		movwf	v_b_in
		movlw	B'10000111'
		PAGESEL	PAGE1
		call	hex_input
		PAGESEL	PAGE2
		movf	v_b_in,0
		movwf	v_fuse_l

		movf	v_comm,0
		xorlw	H'01'
		btfsc	STATUS,Z
		goto	att_113_leave
		movf	v_comm,0
		xorlw	H'02'
		btfsc	STATUS,Z
		goto	att_113_c1
		movf	v_comm,0
		xorlw	H'03'
		btfsc	STATUS,Z
		goto	att_113_save


;input fuse high
att_113_c1	movf	v_fuse_h,0
		movwf	v_b_in
		movlw	B'10010101'
		PAGESEL	PAGE1
		call	hex_input
		PAGESEL	PAGE2
		movf	v_b_in,0
		movwf	v_fuse_h

		movf	v_comm,0
		xorlw	H'01'
		btfsc	STATUS,Z
		goto	att_113_leave
		movf	v_comm,0
		xorlw	H'02'
		btfsc	STATUS,Z
		goto	att_113_c2
		movf	v_comm,0
		xorlw	H'03'
		btfsc	STATUS,Z
		goto	att_113_save

;input fuse extended
att_113_c2	nop

		movf	v_att_chip,0
		xorlw	H'08'
		btfsc	STATUS,Z
		goto	att_113_c3		;no extended fuse for ATtiny26

		movf	v_fuse_e,0
		movwf	v_b_in
		movlw	B'11000111'
		PAGESEL	PAGE1
		call	hex_input
		PAGESEL	PAGE2
		movf	v_b_in,0
		movwf	v_fuse_e

		movf	v_comm,0
		xorlw	H'01'
		btfsc	STATUS,Z
		goto	att_113_leave
		movf	v_comm,0
		xorlw	H'02'
		btfsc	STATUS,Z
		goto	att_113_c3
		movf	v_comm,0
		xorlw	H'03'
		btfsc	STATUS,Z
		goto	att_113_save

;input lock bits
att_113_c3	movf	v_lock_b,0
		movwf	v_b_in
		movlw	B'11010101'
		PAGESEL	PAGE1
		call	hex_input
		PAGESEL	PAGE2
		movf	v_b_in,0
		movwf	v_lock_b

		movf	v_comm,0
		xorlw	H'01'
		btfsc	STATUS,Z
		goto	att_113_leave
		movf	v_comm,0
		xorlw	H'02'
		btfsc	STATUS,Z
		goto	att_113_c7
		movf	v_comm,0
		xorlw	H'03'
		btfsc	STATUS,Z
		goto	att_113_save

		goto	att_113_c7


att_113_save	nop
		call	leave_prog		;exit the programming mode completely
		goto	at_113_wf

att_113_leave	nop
		PAGESEL	PAGE1
		call	w_att113_id
		PAGESEL	PAGE2
		goto	att_001
;XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
;-------------------------------------------------------------------------------
;-------------------------------------------------------------------------------
;    Fuse Maintenance
;    ATtiny2313(A)
;-------------------------------------------------------------------------------
;-------------------------------------------------------------------------------
;XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
;FUSE EXTENDED screen
;incl. reading data from the device
;
att_013		call	read_fuse2313
;we have to read the Fuse data from the device first
		
att_013_1	nop

		PAGESEL	PAGE1
		call	disp_clr1
		call	disp_clr2

att_013y	nop
		PAGESEL	PAGE0
		call	d55
		PAGESEL	PAGE1
		call	w_att013_1		;Fuse EXT 2313A
		PAGESEL	PAGE2


att_013x	clrf	kbd_buffer
		PAGESEL	PAGE0
		call	get_datafk
		PAGESEL	PAGE2

		movf	kbd_buffer,0
		xorlw	H'5A'		;ENTER = read next
		btfsc	STATUS,Z
		goto	att_013_2

		movf	kbd_buffer,0
		xorlw	H'1B'		;S = toggle SELFPRGEN
		btfsc	STATUS,Z
		goto	att_013_1S


		movf	kbd_buffer,0
		xorlw	H'09'		;F10 = write data to the device
		btfsc	STATUS,Z
		goto	at2313_wf

		movf	kbd_buffer,0
		xorlw	H'76'		;ESC = back
		btfsc	STATUS,Z
		goto	att_001

		goto	att_013x
;-------------------------------------------------------------------------------
att_013_1S	movlw	B'00000001'		;bit0 = NOT( bit0 )
		xorwf	v_fuse_e,1

		bsf	v_fuse_mod,D'000'	;data has been modified
		goto	att_013y
;-------------------------------------------------------------------------------
;-------------------------------------------------------------------------------
;FUSE HIGH-1 screen
att_013_2	nop

		PAGESEL	PAGE1
		call	disp_clr1
		call	disp_clr2

att_013_2y	nop
		PAGESEL	PAGE0
		call	d55
		PAGESEL	PAGE1
		call	w_att013_2		;Fuse HIGH-1 2313A
		PAGESEL	PAGE2


att_013_2x	clrf	kbd_buffer
		PAGESEL	PAGE0
		call	get_datafk
		PAGESEL	PAGE2

		movf	kbd_buffer,0
		xorlw	H'5A'		;ENTER = read next
		btfsc	STATUS,Z
		goto	att_013_3

		movf	kbd_buffer,0
		xorlw	H'1B'		;S = toggle SPIEN
		btfsc	STATUS,Z
		goto	att_013_2S

		movf	kbd_buffer,0
		xorlw	H'23'		;D = toggle DWEN
		btfsc	STATUS,Z
		goto	att_013_2D

		movf	kbd_buffer,0
		xorlw	H'24'		;E = toggle EESAVE
		btfsc	STATUS,Z
		goto	att_013_2E

		movf	kbd_buffer,0
		xorlw	H'1D'		;W = toggle WDTON
		btfsc	STATUS,Z
		goto	att_013_2W

		movf	kbd_buffer,0
		xorlw	H'09'		;F10 = write data to the device
		btfsc	STATUS,Z
		goto	at2313_wf

		movf	kbd_buffer,0
		xorlw	H'76'		;ESC = back
		btfsc	STATUS,Z
		goto	att_001

		goto	att_013_2x
;-------------------------------------------------------------------------------
att_013_2S	movlw	B'00100000'		;bit5 = NOT( bit5 )
		xorwf	v_fuse_h,1

		bsf	v_fuse_mod,D'000'	;data has been modified
		goto	att_013_2y
;-------------------------------------------------------------------------------
att_013_2D	movlw	B'10000000'		;bit7 = NOT( bit7 )
		xorwf	v_fuse_h,1

		bsf	v_fuse_mod,D'000'	;data has been modified
		goto	att_013_2y
;-------------------------------------------------------------------------------
att_013_2E	movlw	B'01000000'		;bit6 = NOT( bit6 )
		xorwf	v_fuse_h,1

		bsf	v_fuse_mod,D'000'	;data has been modified
		goto	att_013_2y
;-------------------------------------------------------------------------------
att_013_2W	movlw	B'00010000'		;bit4 = NOT( bit4 )
		xorwf	v_fuse_h,1

		bsf	v_fuse_mod,D'000'	;data has been modified
		goto	att_013_2y

;-------------------------------------------------------------------------------
;-------------------------------------------------------------------------------
;FUSE HIGH-2 screen
att_013_3	nop

		PAGESEL	PAGE1
		call	disp_clr1
		call	disp_clr2

att_013_3y	nop
		PAGESEL	PAGE0
		call	d55
		PAGESEL	PAGE1
		call	w_att013_3		;Fuse HIGH-2 2313A
		PAGESEL	PAGE2


att_013_3x	clrf	kbd_buffer
		PAGESEL	PAGE0
		call	get_datafk
		PAGESEL	PAGE2

		movf	kbd_buffer,0
		xorlw	H'5A'		;ENTER = read next
		btfsc	STATUS,Z
		goto	att_013_4

		movf	kbd_buffer,0
		xorlw	H'32'		;B = inc BODLEVEL
		btfsc	STATUS,Z
		goto	att_013_3B

		movf	kbd_buffer,0
		xorlw	H'2D'		;R = toggle RSTDISBL
		btfsc	STATUS,Z
		goto	att_013_3R

		movf	kbd_buffer,0
		xorlw	H'09'		;F10 = write data to the device
		btfsc	STATUS,Z
		goto	at2313_wf

		movf	kbd_buffer,0
		xorlw	H'76'		;ESC = back
		btfsc	STATUS,Z
		goto	att_001

		goto	att_013_3x
;-------------------------------------------------------------------------------
att_013_3B	movf	v_fuse_h,0
		andlw	B'00000110'
		movwf	v_tmp1
		rrf	v_tmp1,1
		incf	v_tmp1,1
		rlf	v_tmp1,0
		andlw	B'00000110'
		movwf	v_tmp1
		movlw	B'11111001'
		andwf	v_fuse_h,0
		addwf	v_tmp1,0
		movwf	v_fuse_h

		bsf	v_fuse_mod,D'000'	;data has been modified
		goto	att_013_3y

;-------------------------------------------------------------------------------
att_013_3R	movlw	B'00000001'		;bit0 = NOT( bit0 )
		xorwf	v_fuse_h,1

		bsf	v_fuse_mod,D'000'	;data has been modified
		goto	att_013_3y

;-------------------------------------------------------------------------------
;-------------------------------------------------------------------------------
;FUSE LOW screen
att_013_4	nop

		PAGESEL	PAGE1
		call	disp_clr1
		call	disp_clr2

att_013_4y	nop
		PAGESEL	PAGE0
		call	d55
		PAGESEL	PAGE1
		call	w_att013_4		;Fuse LOW 2313A
		PAGESEL	PAGE2


att_013_4x	clrf	kbd_buffer
		PAGESEL	PAGE0
		call	get_datafk
		PAGESEL	PAGE2

		movf	kbd_buffer,0
		xorlw	H'5A'		;ENTER = read next
		btfsc	STATUS,Z
		goto	att_013_1

		movf	kbd_buffer,0
		xorlw	H'23'		;D
		btfsc	STATUS,Z
		goto	att_013_4D

		movf	kbd_buffer,0
		xorlw	H'44'		;O
		btfsc	STATUS,Z
		goto	att_013_4O

		movf	kbd_buffer,0
		xorlw	H'1B'		;S
		btfsc	STATUS,Z
		goto	att_013_4S

		movf	kbd_buffer,0
		xorlw	H'2C'		;T
		btfsc	STATUS,Z
		goto	att_013_4T

		movf	kbd_buffer,0
		xorlw	H'09'		;F10 = write data to the device
		btfsc	STATUS,Z
		goto	at2313_wf

		movf	kbd_buffer,0
		xorlw	H'76'		;ESC = back
		btfsc	STATUS,Z
		goto	att_001

		goto	att_013_4x
;-------------------------------------------------------------------------------
att_013_4D	movlw	B'10000000'
		xorwf	v_fuse_l,1

		bsf	v_fuse_mod,D'000'	;data has been modified
		goto	att_013_4y
;-------------------------------------------------------------------------------
att_013_4O	movlw	B'01000000'
		xorwf	v_fuse_l,1

		bsf	v_fuse_mod,D'000'	;data has been modified
		goto	att_013_4y
;-------------------------------------------------------------------------------
att_013_4S	movf	v_fuse_l,0
		andlw	B'00001111'
		movwf	v_tmp1
		incf	v_tmp1,0
		andlw	B'00001111'
		movwf	v_tmp1
		movlw	B'11110000'
		andwf	v_fuse_l,0
		addwf	v_tmp1,0
		movwf	v_fuse_l

		bsf	v_fuse_mod,D'000'	;data has been modified
		goto	att_013_4y
;-------------------------------------------------------------------------------
att_013_4T	movf	v_fuse_l,0
		andlw	B'00110000'
		movwf	v_tmp1
		rrf	v_tmp1,1
		rrf	v_tmp1,1
		rrf	v_tmp1,1
		rrf	v_tmp1,1
		incf	v_tmp1,0
		rlf	v_tmp1,1
		rlf	v_tmp1,1
		rlf	v_tmp1,1
		rlf	v_tmp1,1
		andlw	B'00110000'
		movwf	v_tmp1
		movlw	B'11001111'
		andwf	v_fuse_l,0
		addwf	v_tmp1,0
		movwf	v_fuse_l

		bsf	v_fuse_mod,D'000'	;data has been modified
		goto	att_013_4y
;-------------------------------------------------------------------------------
;-------------------------------------------------------------------------------
at2313_wf	goto	at_113_wf
;-------------------------------------------------------------------------------
;WRITE FUSE bits
at_113_wf	nop
		PAGESEL	PAGE1
		call	disp_clr1
		call	disp_clr2
		call	w_wf_info
		movlw	B'11000000'
		call	send_comm
		PAGESEL	PAGE2
;--------------------------------------------------------------
;start programming mode
		call	att_init_prg
		movlw	B'01000000'		;ATtiny command: write Fuse
		movwf	v_byte
		call	att_sgn_open
;Fuse Low -----------------------------------------------
		movf	v_fuse_l,0
		movwf	v_att_add


		PAGESEL	PAGE1
		movwf	kbd_buffer
		call	write_hex
		call	wm_minus
		PAGESEL	PAGE2


		call	att_dat_L
		bcf	PORTD,D'007'		;BS1
		bcf	PORTB,D'004'		;BS2
		bcf	PORTC,D'001'		;BS2 (28-pin)

		bcf	PORTD,D'006'		;-WR := 0
		PAGESEL	PAGE0
		call	d2			;wait a little bit longer - just for sure (we are currently ignoring RDY/-BSY)
		PAGESEL	PAGE2
		bsf	PORTD,D'006'

		PAGESEL	PAGE0
		call	d55
		PAGESEL	PAGE2
;Fuse High ----------------------------------------------
		movlw	B'01000000'		;ATtiny command: write Fuse
		movwf	v_byte
		call	att_sgn_open

		movf	v_fuse_h,0
		movwf	v_att_add


		PAGESEL	PAGE1
		movwf	kbd_buffer
		call	write_hex
		call	wm_minus
		PAGESEL	PAGE2


		call	att_dat_L
		bsf	PORTD,D'007'		;BS1
		bcf	PORTB,D'004'		;BS2
		bcf	PORTC,D'001'		;BS2 (28-pin)

		bcf	PORTD,D'006'		;-WR := 0
		PAGESEL	PAGE0
		call	d2			;wait a little bit longer - just for sure (we are currently ignoring RDY/-BSY)
		PAGESEL	PAGE2
		bsf	PORTD,D'006'

		PAGESEL	PAGE0
		call	d55
		PAGESEL	PAGE2


		movf	v_att_chip,0
		xorlw	H'08'
		btfsc	STATUS,Z
		goto	att_113_wfc4		;no ext. fuse for ATtiny26

;Fuse Ext ----------------------------------------------
		movlw	B'01000000'		;ATtiny command: write Fuse
		movwf	v_byte
		call	att_sgn_open

		movf	v_fuse_e,0
		movwf	v_att_add


		PAGESEL	PAGE1
		movwf	kbd_buffer
		call	write_hex
		call	wm_minus
		PAGESEL	PAGE2


		call	att_dat_L
		bcf	PORTD,D'007'		;BS1
		bsf	PORTB,D'004'		;BS2
		bsf	PORTC,D'001'		;BS2 (28-pin)

		bcf	PORTD,D'006'		;-WR := 0
		PAGESEL	PAGE0
		call	d2			;wait a little bit longer - just for sure (we are currently ignoring RDY/-BSY)
		PAGESEL	PAGE2
		bsf	PORTD,D'006'

		PAGESEL	PAGE0
		call	d55
		PAGESEL	PAGE2

;Lock Bits ----------------------------------------------
att_113_wfc4	nop
		movlw	B'00100000'		;ATtiny command: Lock Bits
		movwf	v_byte
		call	att_sgn_open

		movf	v_lock_b,0
		movwf	v_att_add


		PAGESEL	PAGE1
		movwf	kbd_buffer
		call	write_hex
		PAGESEL	PAGE2


		call	att_dat_L
		bcf	PORTD,D'007'		;BS1
		bcf	PORTB,D'004'		;BS2
		bcf	PORTC,D'001'		;BS2 (28-pin)

		bcf	PORTD,D'006'		;-WR := 0
		PAGESEL	PAGE0
		call	d2			;wait a little bit longer - just for sure (we are currently ignoring RDY/-BSY)
		PAGESEL	PAGE2
		bsf	PORTD,D'006'

		PAGESEL	PAGE0
		call	d55
		PAGESEL	PAGE2
;--------------------------------------------------------------
		call	leave_prog		;exit the programming mode completely

		goto	att_113			;display fuse
;

;-------------------------------------------------------------------------------
;-------------------------------------------------------------------------------
;-------------------------------------------------------------------------------
d55p2		movlw	D'002'
		movwf	TIMER2
d55p2_loop	decfsz	TIMER2,F
		goto	d55p2_loop
		return
;-------------------------------------
d55_0		movlw	D'255'
		movwf	TIMER2
d55_0_loop	decfsz	TIMER2,F
		goto	d55_0_loop
		return
;-------------------------------------------------------------------------------
;-------------------------------------------------------------------------------
;XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX

		END